#define DTVERSION 20 		/* version number (int) */
/* changelog 
version	date		comments
2
3		6.5.92	changed dtclose() to use FSF when closed in
				middle of file
				put in lseek(fd,0,0) in dtopen routine to
				get round 2Gb file limit
4		30.7.92	added support for reading fixed block length
				format
5		9.12.92	add support for writing fixed blocks
				add dtlabel, rewrite dtopen to skip one file
				at a time
6		10.12.92	convert to ANSI c (gcc), add error handling
				in dtwrite
7		21.1.93	added dtrewind()
8		30.4.93	added dtskip()
9		6.7.93	added fortran callable routines and bug fixes
10		7.7.93	fixed fortran callable routines

20              9.2.95  mods for multi-streams (jrc)

*/



#define READ_MODE  0
#define WRITE_MODE 1

#undef FALSE
#undef TRUE


enum bool { FALSE, TRUE };


#define HEAD 0
#define TAIL 1

#define NOPROTECT ' '         /* space if no protection             */
#define REC_VARIABLE 'D'      /* Records are preceeded by a 4-digit *
                               * zero-filled count of their length, *
                               * including the 4 digits themselves  */
#define REC_FIXED 'F'		/* Fixed block length			  */
#define HDR_LEN 80	/* all header blocks are 80 bytes     */

/* error codes */
#define SUCCESS                 0
#define MOUNT_UNKNOWN_MODE     -1
#define MOUNT_OPEN_ERROR       -2
#define MOUNT_INVALID_DEVICE   -3
#define MOUNT_WRONG_TAPE       -4
#define MOUNT_NOT_ANSI_FORMAT  -5
#define MOUNT_INVALID_VOL      -6
#define MOUNT_UNINITIALISED    -7
#define FOPEN_VOL_NOT_MOUNTED -10
#define FOPEN_EOV	      -11
#define FOPEN_FAULTY_STRUCT   -12
#define FOPEN_FILE_OPEN       -13
#define FOPEN_NOT_FOUND		-14
#define FOPEN_UNKNOWN_MODE	-15
#define FOPEN_REC_SIZE		-16
#define FOPEN_BLOCK_SIZE	-17
#define FCLOSE_FILE_CLOSED	-20
#define FCLOSE_FAULTY_STRUCT	-21
#define FCLOSE_UNKNOWN_MODE	-22
#define LABEL_INVALID_VOL	-25
#define LABEL_INVALID_DEVICE	-26
#define LABEL_OPEN_ERROR	-27
#define WRITE_REC_SIZE		-30
#define WRITE_ERROR		-31
#define WRITE_NOT_OPEN		-32
#define READ_REC_SIZE		-40
#define READ_ERROR		-41
#define READ_NOT_OPEN		-42
#define TAPE_ERROR		-50
#define REW_DEVICE		-60
#define REW_FILE	       	-61
#define REW_ERROR	       	-62
#define SKIP_NEGATIVE		-70
#define SKIP_FILE	       	-71
#define SKIP_FORMAT		-72
#define SKIP_ERROR		-73
#define STREAM_NOT_CONNECTED    -80
#define STREAM_ID_INVALID       -81
#define STREAM_IN_USE           -82


/*
 * VOLUME 1 Describes the owner of and access to the volume. 
 */

struct ansi_vol1 {
    char            header[4];	/* VOL1 */
    char            label[6];	/* volume name, A-Z0-9 */
    char            access;	/* access code */
    char            __ignored1[20];	/* reserved, space fill */
    char            __ignored2[6];	/* reserved, space fill */
    char            owner[14];	/* user name */
    char            __ignored3[28];	/* reserved, space fill */
    char            ansi_level;	/* ansi standard level, always 3 */
};

/*
 * VOLUME 2 optional, ignored by this software. 
 */

/*
 * HEADER 1 Begins basic description of a tape file. 
 */

struct ansi_hdr1 {
    char            header[4];	/* HDR1 */
    char            name[17];       /* rightmost 17 characters of filename */
    char            fileset[6];	/* should be same as volume id of 1st
                               	 * tape */
    char            volume_num[4];	/* number of volume in set */
    char            file_num[4];	/* number of file on tape */
    char            gen[4];		/* generation number */
    char            genver[2];	/* generation version number */
    char            created[6];	/* date of creation, "bYYDDD" */
    char            expires[6];	/* expiration date "bYYDDD" */
    char            access;		/* protection, space = unprotected */
    char            blockcount[6];	/* zero in header */
    char            tapesys[13];	/* name of software creating tape */
    char            __ignored[7];	/* reserved, space fill */
};

/*
 * HEADER 2 Record format, record size, block size. 
 */

struct ansi_hdr2 {
    char            header[4];	/* HDR2 */
    char            recfmt;		/* record format */
    char            blocklen[5];	/* file tape block length */
    char            reclen[5];	/* file logical record length */
    char            density;		/* density/recording mode. Should be 2
                            		 * for 800 bpi, 3 for 1600 bpi. */
    char            vol_switch;	/* 0, or 1 if this file was continued */
    char            job[17];		/* job name (8), /, job step (8) of
                            		 * creator */
    char            recording[2];	/* parity/conversion, space
                                 	 * fill */
    char            carriage_control;
    char            alignment;	/* reserved, space fill */
    char            blocked_records;/* 'B' if records are blocked */
    char            __ignored2[11];
    char            block_offset[2];/* ignore 1st n char of each
                                     * block */
    char            __ignored3[28];
};

/* HEADER 3 - ignore */

struct ansi_hdr3 {
    char            header[4];	/* HDR3 */
    char            os_reserved[76];	/* reserved to OS, space fill */
};

/* HEADER 4 - the rest of the filename if its longer than 17 characters. */

struct ansi_hdr4 {
    char            header[4];	/* HDR4 */
    char            name2[63];	/* leftmost char, if name is more than
				 * 17 characters long. */
    char            unknown[2];	/* fill with 00 */
    char            __ignored[11];
};


/*
 * FORMAT NOTES 
 *
 * 1.  The tape always starts with a VOL1 record.  Other volume records
 * are optional.  We ignore them, and do not generate them on output. 
 *
 * 2.  Each file starts with a set of HDRn records.  The headers are
 * followed by a tape mark. 
 *
 * 3.  The data is written to the tape in fixed-length blocks.  Valid ANSI
 * block lengths are 18 to 2048 bytes, although this software supports up to
 * 16k blocks. The block does not have to be full.  If the block is not
 * completely filled, the remainder of the block is padded with
 * circumflex (^) characters.  The file is followed by a tape mark. 
 *
 * 4.  Trailing records are written.  Either EOF or EOV records may be
 * used. The record formats are the same as the HDR records, except: 
 *
 * a. the "HDR" is replaced by "EOF" or "EOV". 
 *
 * b. the blockcount field in EOF1/EOV1 contains the actual number of
 * physical tape blocks written in the preceding file section.  In the
 * HDR1 record, the blockcount is 000000. 
 *
 * 5.  A tape file may extend across a volume boundary.  If it does, EOV
 * labels are used, indicating that another tape must be read.  If the
 * file ends here, an EOF label is used. EOV not implememnted. 
 *
 * 6.  After the EOF records, a tape mark is written.  This separates the
 * trailers from the headers for the next file.  At the end of the
 * tape, two tape marks are written. 
 */

struct file_data {
  char	filename[80];
  char	volume[6];
  int	blen;		/* physical block length */
  int	rlen;		/* record length, <= block length */
  int	blocks;	        /* number of blocks written/read */
  char	format;	        /* record format REC_FIXED or REC_VARIABLE */
  enum bool blocking;	/* blocking, TRUE if as many records as will fit are in each block, or FALSE if only one record per block */
  enum bool hdr34;	/* TRUE if hdr3 and hdr4 records exist */
};

struct device_info {
  int  fd;		        /* file descriptor for device		*/
  enum bool vol_mounted;	/* true if volume mounted on device */
  enum bool just_mounted;	/* true if at start of tape		*/
  enum bool file_open;          /* true if a file is currently open */
  struct file_data fdata;	/* info about current file		*/
};

