/*******************************************************************************
* spectrum.h        describes the Unified Spectrum Format  see EDOC061
*/

/*
 * most things have probably been defined in spectfio.h
 *
 * some offsets may be declared as char * if absolute addressing is used
 */

/* $Header: /eurogam/spectrumServer/RCS/spectrum.h,v 2.1 1992/04/30 09:06:24 scl Exp scl $
*/

/* $Log: spectrum.h,v $
 * Revision 2.1  1992/04/30  09:06:24  scl
 * New revision. last revision frozen at 1.x.
 *  Implements new string management, new memory management
 *
 * Revision 1.17  1992/03/31  10:13:39  jcd
 * frozen 31 March 1992
 *
 * Revision 1.17  1992/03/31  10:13:39  jcd
 * frozen 31 March 1992
 *
 * Revision 1.16  1991/11/27  17:17:29  scl
 * frozen revision for 1st distribution to France
 *
 * Revision 1.15  1991/11/15  11:13:47  scl
 * some strings changed to opaques
 *
 * Revision 1.14  1991/11/15  10:35:20  jcd
 * *** empty log message ***
 *
 * Revision 1.13  1991/11/11  11:38:46  jcd
 * bug hunting
 *
 * Revision 1.12  1991/11/08  10:20:37  jcd
 * *** empty log message ***
 *
 * Revision 1.11  1991/11/01  14:22:06  jcd
 * *** empty log message ***
 *
 * Revision 1.10  1991/10/31  14:35:13  jcd
 * *** empty log message ***
 *
 * Revision 1.9  1991/10/31  14:33:47  scl
 * structure for strings handling added
 *
 * Revision 1.7  1991/10/29  15:33:33  jcd
 * *** empty log message ***
 *
 * Revision 1.6  1991/10/24  15:28:31  jcd
 * *** empty log message ***
 *
 * Revision 1.5  1991/10/04  13:36:47  scl
 * *** empty log message ***
 *
 * Revision 1.4  1991/09/10  15:52:07  jcd
 * *** empty log message ***
 *
 * Revision 1.3  1991/08/30  14:15:17  jcd
 * *** empty log message ***
 *
 * Revision 1.2  1991/08/30  13:17:20  jcd
 * maximum No. of online spectra set at 32
 *
 * Revision 1.1  1991/08/28  13:17:32  jcd
 * Initial revision
 *
*/

#ifndef INCspectrumh
#define INCspectrumh

#ifndef STATUS
#define STATUS int
#endif

#ifndef VOID
#define VOID void
#endif

#ifndef IMPORT
#define IMPORT extern
#endif

#ifndef LOCAL
#define LOCAL static
#endif

#ifndef NULL
#define NULL 0
#endif

#ifdef UNIX
#ifndef min
#define min(a,b) (((a)<(b))?(a):(b))
#endif
#ifndef max
#define max(a,b) (((a)>(b))?(a):(b))
#endif
#define STATUS int
#endif

#ifdef UNIXVW
#ifndef min
#define min(a,b) (((a)<(b))?(a):(b))
#endif
#ifndef max
#define max(a,b) (((a)>(b))?(a):(b))
#endif
#define STATUS int
#endif

#define BIT_16 0x8000
#define N_BITS 1

#define SPECTRUM_ARRAY 0
#define MAX_ARRAY 2
#define MAX_LAYOUT 1
#define MAX_TYPE 6

#define MAX_SPECS 32		   /* maximum number of online spectra	      */

#define MAGIC_NUMBER 412900921     /* to identify a file as a spectrum        */
#define VERSION 1                  /* this version of the format              */
#define UNUSED (~0)                /* unused offset => NULL pointer           */

#define MAX_DIM 8                  /* maximum number of dimensions            */
#define MAX_INF 32                 /* maximum number of information pointers  */
#define MAX_DAD 2                  /* maximum number of array descriptors     */
#define TIM_LEN 20                 /* length of time and date array           */
#define NAM_LEN 32                 /* length of name array                    */

#define BLOCK_SIZE 256		   /* length of a block for space allocation  */
#define HDR_LENGTH 512		   /* length of header in bytes  */
#define HDR_SIZE (HDR_LENGTH/BLOCK_SIZE)  /* length of header in blocks	      */
#define HDR_LOC 0		   /* start block for header		      */

#define MAX_UDP_DATA 8192	   /* max bytes that can be sent */
#define BUFFER_SIZE MAX_UDP_DATA

typedef enum STRING_TYPE_OPT {
	TYPE_INFORMATION	= 0,
	TYPE_ANNOTATION		= 1,
	TYPE_CALIBRATION	= 2,
	TYPE_EFFICIENCY		= 3,
}STRING_TYPE;

typedef enum SAS_DIRECTIONS {
	SAS_TO_NET,		   /* directions of conversion for Resizing   */
	SAS_FROM_NET,
}SAS_DIRECTION;

typedef enum ARRAY_TYPE_OPT {
	TYPE_8_BIT_UNSIGNED  = 0,
	TYPE_8_BIT_SIGNED    = 1,
	TYPE_16_BIT_UNSIGNED = 2,
	TYPE_16_BIT_SIGNED   = 3,
	TYPE_32_BIT_UNSIGNED = 4,
	TYPE_32_BIT_SIGNED   = 5,
	TYPE_32_BIT_FLOAT    = 6,
}ARRAY_TYPE;

typedef enum ARRAY_LAYOUT_OPT {
	AT_MATRIX = 0,
	AT_HALF_MATRIX = 1,
}ARRAY_LAYOUT;

typedef struct dataArrayDescriptor /* data array format description           */
{
    unsigned int layout;		   /* array layout			      */
    unsigned int type;                   /* array type                              */
    unsigned int reserved[2];            /* reserved for future use                 */
    unsigned int pointer;                /* offset from count space base            */
} DAD;

typedef struct spaceDescriptor     /* string/counts space description         */
{
    unsigned int base;                   /* offset from spectrum base address       */
    unsigned int free;                   /* offset to first unused location         */
    unsigned int top;                    /* offset to last available location       */
} SPACE;

typedef char  PADDING[76];

typedef struct unifiedSpectrumFormatHeader /* spectrum header                 */
{
    unsigned int magic;                          /* magic number                    */
    unsigned int version;                        /* header version number           */
    char   name         [NAM_LEN];         /* spectrum name                   */
    unsigned int dimension;                      /* number of dimensions            */
    char   creation     [TIM_LEN];         /* time and date of creation       */
    char   modification [TIM_LEN];         /* time and date of modification   */
    unsigned int base         [MAX_DIM];         /* first channel of each dimension */
    unsigned int range        [MAX_DIM];         /* number of channels              */
    unsigned int information  [MAX_INF];         /* offset to information string    */
    unsigned int annotation   [MAX_DIM];         /* offset to annotation string     */
    unsigned int calibration  [MAX_DIM];         /* offset to calibration string    */
    unsigned int efficiency   [MAX_DIM];         /* offset to efficiency string     */
    DAD    array        [MAX_DAD];         /* data array format descriptions  */
    SPACE  string;                         /* description of string space     */
    SPACE  count;                          /* description of counts space     */
    PADDING unused;             	   /* pad out to 512 bytes            */
} HEADER, *HEADER_PTR;

typedef struct spectrumContents
{
	unsigned int offset;
	unsigned int length;   /* length in bytes */
	char  *data;
} SPECTRUM, *SPECTRUM_PTR;

#ifndef UNIX /* VXWORKS */
#define	EOLIST	NULL			      /* end of a SPECLIST in VXWORKS */
typedef struct spectrumList		   /* items in list of online spectra */
{
	struct spectrumList *next;
	struct spectrumList *prev;
	HEADER_PTR	header;		   /* the spectrum header	      */
	char		name[NAM_LEN];	   /* as in HEADER                    */
} SPECLIST, *SPECLIST_PTR;
#endif

enum flagValues
{
	FLAG_OK = 0,
	FLAG_OVERFLOW = (~0)
};

extern STATUS sasPathValid (char *path);
extern STATUS sasCreateSpectrum (char *path, unsigned int dim, unsigned int *base, unsigned int *range,
              ARRAY_LAYOUT layout1, ARRAY_TYPE type1, ARRAY_LAYOUT layout2, ARRAY_TYPE type2);
extern STATUS sasDeleteSpectrum (char *path);
extern HEADER_PTR sasLookupSpectrum (char *path, void **specDesc);
extern STATUS sasReadString (char *path, STRING_TYPE type, unsigned int number, unsigned int *pLength, char **pString);
extern STATUS sasWriteString (char *path, STRING_TYPE type, unsigned int number, unsigned int dataLen, char *dataVal);
extern char * sasSpectrumName (char *path, unsigned int number);
extern char * sasReadSpectrum (char *path, unsigned int number, unsigned int offset, unsigned int count, unsigned int *base, unsigned int *range, unsigned int *size, unsigned int type, char *buffer, unsigned int *flag);
extern char * sasReadSpectrum2 (char *path, unsigned int number, unsigned int offset, unsigned int count, unsigned int *base, unsigned int *range, unsigned int *size, unsigned int type, char *buffer, unsigned int *flag);
extern STATUS sasWriteSpectrum (char *path, unsigned int number, unsigned int offset, unsigned int count, unsigned int *base, unsigned int *range, unsigned int type, char *data, unsigned int *flag);
extern STATUS sasChangeArrayOptions (char *path, unsigned int number, unsigned int layout, unsigned int type);
extern char * sasGetWindow (char *spectrum, char *array, HEADER_PTR header, unsigned int type, unsigned int typeout, unsigned int *base, unsigned int *range, int *size, unsigned int offset, unsigned int count, unsigned int *flag);
extern char * sasGetWindow2 (char *spectrum, char *array, HEADER_PTR header, unsigned int type, unsigned int typeout, unsigned int *base, unsigned int *range, int *size, unsigned int offset, unsigned int count, unsigned int *flag);
extern STATUS sasPutWindow (char *spectrum, char *data, HEADER_PTR header, unsigned int specType, unsigned int type, unsigned int *base, unsigned int *range, unsigned int offset, unsigned int count, unsigned int *flag);
extern char * sasRetype (char *data, char *buffer, int count, int from, int to, unsigned int *flag, int forceCopy);
extern char * sasRetype1 (char *inData, char *outData, int from, int to, unsigned int *flag);
extern STATUS sasCheckRange (char *path, unsigned int *usr_base, unsigned int *usr_range, unsigned int *hdr_base, unsigned int *hdr_range, unsigned int dim, unsigned int *count);

#endif /* INCspectrumh */
