set comtitle "TDR Event Builder Exec"
set com_icontitle "TDR EB"
set execpid 0

set userid support

set execpidfile /tmp/.pid_TDRexec

set execloaded 0

#set termtype cmdtool
set termtype xterm

# Filing ...

proc fopen {name {attr r} } {
    set err [catch {set f [open $name $attr] } m]
    if {$err} {
	return 0
    }
    return $f
}

proc isComment? {s} {
    if { $s =="" } {
	set b 1
    } else {
	set b [expr { [string index $s 0]=="#" } ]
    }
    return $b
}

proc read-file {f} {
    set buf ""
    if {[file exists $f]} {
	set fp [fopen $f r]
	if {$fp==0} return ""
	while 1 {
	    set t [gets $fp]
	    if {[eof $fp]} {break}
	    if { ! [isComment? $t] } { lappend buf $t }
	}
	close $fp
    }
    return $buf
}

proc write-file {f buf} {
    set fp [fopen $f w+]
    foreach line $buf {
	puts $fp $line
    }
    close $fp
    set-footer "Saved to $f"
}

# Experimenting ...

proc create-experiment {} {
    global env

    if {$env(EXPUSEENDIAN) == "true"} then {set endian $env(ENDIAN)} else {set endian "big"}

    set exptdir [set env(HOME)]/.egexperiments
    if {![file exists $exptdir]} {
	set exptdir /MIDAS/experiments
	if {![file writable $exptdir]} {
	    set exptdir [set env(HOME)]/.egexperiments
	    file mkdir $exptdir
	}
    }

    set exptname TDR.[exec uname -n]
    set exptnamedir [set exptdir]/[set exptname]

    if {![file exists $exptnamedir]} {
	puts "Creating experiment directory $exptnamedir"
	file mkdir $exptnamedir
	if {$endian == "little"} {
	    file mkdir $exptnamedir/little
	    set env(EXPERIMENTDIR) $exptnamedir/little
	} {
	    set env(EXPERIMENTDIR) $exptnamedir
	}
	set env(EXPERIMENTBASE) $exptnamedir

	if {![file exists [set env(EXPERIMENTDIR)]/capabilities]} {
	    puts "Creating experiment directory capability file"
	    :experiment new :path [set env(EXPERIMENTDIR)] :name capabilities
	}
	attach-shm-resource
    } {
	puts "Connecting to experiment directory $exptnamedir"
	if {$endian == "little"} {
	    if {![file exists $exptnamedir/little]} {
		file mkdir $exptnamedir/little
	    }
	    set env(EXPERIMENTDIR) $exptnamedir/little
	} {
	    set env(EXPERIMENTDIR) $exptnamedir
	}
	set env(EXPERIMENTBASE) $exptnamedir

	puts "Connecting to experiment base directory $exptdir"

	if {![file exists [set env(EXPERIMENTDIR)]/capabilities]} {
	    puts "Creating experiment directory capability file"
	    :experiment new :path [set env(EXPERIMENTDIR)] :name capabilities
	}
    }

    inform ex auto-join
    claim-shm-resource

}

proc add_apparatus {} {
    global env apparatus_name apparatus_host apparatus_role apparatus_type apparatus_port
    
#    check for and is necessary create a temporary directory
    if {[midas-check-for-temp] == 0} {return}

    set directory [file join $env(MIDASTempDir) tcl[pid]]
    set file $apparatus_name

#    create apparatus file
    set z [catch {open [file join $directory $file] w} f]
    if {$z != 0} then {midas-warning "Unable to write [file join $directory $file]\n$f"; return}

    puts $f "((name $apparatus_name)"
    puts $f " (host $apparatus_host)"
    puts $f " (role $apparatus_role)"
    puts $f " (type $apparatus_type)"
    puts $f " (ipaddress 123456)"
    puts $f " (port $apparatus_port))"

    close $f

#    attach apparatus
   :apparatus attach :path $directory :name $file

    puts "[set apparatus_type] Server resource attached"
}

proc attach-shm-resource {} {
    global apparatus_name apparatus_host apparatus_role apparatus_type apparatus_port

    set host [exec uname -n]
    set apparatus_name [set host]shm
    set apparatus_host [exec uname -n]
    set apparatus_role sort
    set apparatus_type spectrum
    set apparatus_port 10235

    add_apparatus
}

proc claim-shm-resource {} {
    global apparatus_name env

   :spectra auth :user $env(USER) :pass ""
    set host [exec uname -n]
    set apparatus_name [set host]shm
    :apparatus claim :name $apparatus_name
    puts "ShmSas Server resource $apparatus_name claimed"
}

proc is-shm-server-resource ps {
  foreach p $ps {
      case [first $p] in {
         name {set name [second $p]}
         host {set host [second $p]}
         port {set port [second $p]}
         type {set type [second $p]}
         role {set role [second $p]}
         apparatus {set app [second $p]}
         ipaddress {set ipaddress [second $p]}
         capability {set cap [second $p]}
         flags {set flags [second $p]}
         default {} 
         }
      }
  if { $type=="spectrum" && $port == "10235" && [string match "\[0\]*" $cap]} then {
    return $role
    } \
  else {
    return ""
    }
  }

proc make-shm-resource-list rs {
   global debug
   set ll {}
   foreach r $rs {
      set rget [is-shm-server-resource $r]
      if {$debug} then {puts stderr "...rget=$rget"}
      if {[llength $rget]!=0} then {lappend ll $rget}
      }
   if {$debug} {puts stderr "shm resource list=$ll"}
   return $ll
   }

proc check-resources {} {
   global debug
   global env
   if {$debug} then {puts stderr "check-resources"}

   set num_init_errors 0
puts "111"
   set resources {}
   set shmservernames {}
   set z [catch {set exp $env(EXPERIMENTDIR)} m]
   if {$z==0} {
puts "222"
       if {[file tail $env(EXPERIMENTDIR)] == "NULL"} {
	   puts "Cannot use experiment NULL - changing ..."
	   inform ex click-leave
	   create-experiment
       } {
	   set z [catch {eg "(experiment ?resources)"} m]
	   if {$z==0} {
	       set resources $m
puts "333 $m"
	       set shmservernames [make-shm-resource-list $resources]
puts "444 $shmservernames"
	       if {[llength $shmservernames]==0} {
		   set absent_proc1 "   no shmsas server claimed"
		   attach-shm-resource
	       } {
		   set absent_proc1 ""
		   puts "ShmSas Server resource found"
	       }
	       claim-shm-resource
	       
	   } {
	       set absent_proc1 "   dir but no experiment connected"
	       create-experiment
	   }
       }

     } {
	 set absent_proc1 "   no experiment connected"
	 create-experiment
     }

   if {$debug} {puts stderr "num_init_errors=$num_init_errors"}
   case $num_init_errors {
      {1} {display-midas-error-report self "" \
                   "TDR initialisation error:\n" $absent_proc1
	  return 0
      }
      default {return 1}
      }

   }

# Utilities ...

proc confirm {m action} {
   global debug
   set i [tk_dialog [self frame-name].quitdialog "[self frame-title] Confirmation" $m  "warning" 1 "Cancel" "Confirm"]
   update idletasks
   if {$i} then $action
   if {$debug} then {puts stderr "confirm: $m choice $i"}
   }

proc complete-connect {} {
    frame-idle
    set-footer "Re-connected with no message window"
}

proc loadexecprog {bindir prog} {
    global execpidfile execpid execloaded host

    set thishost [exec uname -n]
set thishost nndhcp052

    if {$host != "localhost" && $host != $thishost} {
	complete-load $bindir $prog
	return
    }

    if {$execpid == ""} {set execpid 0}

    if {$execpid} {
	puts "execpid not zero - returning"
	return
    }

    if {![check-resources]} {
	puts "check-resources returned 0 ..."
	return
    }

    if {[file exists $execpidfile]} {
	if {[file owned $execpidfile]} {
	    set z [catch {set pid [read-file $execpidfile]} m]
	    if {!$z} {
		set execpid $pid
		confirm "  TDR program running\nExit & Reload?" "complete-reload $bindir $prog"
		frame-idle
		if {$execloaded} {return}
		confirm "  TDR program running\nReConnect?" "complete-connect"
		set execpid 0
		frame-idle
		return
	    }
	} {
	    set-footer "Another user already running"
	    return
	}
    }
    
    complete-load $bindir $prog
}

proc complete-reload {bindir prog} {
    complete-exit $bindir $prog
    complete-load $bindir $prog
}

proc complete-load {bindir prog} {
    global env debug execpid execpidfile termtype comtitle com_icontitle
    global soap_server host port execloaded userid

    if {$debug} {puts stderr "Starting TDR tasks ..."}

    set libdir [set env(LD_LIBRARY_PATH)]

    set ctitle "[set comtitle] [set prog]"

    set thishost [exec uname -n]

    if {$host == "localhost" || $host == $thishost} {
	set xpos 150
	if {$termtype == "cmdtool"} {
	    set execcomwindow "exec nohup cmdtool -M 10000000 -fn 8x13 -Wp $xpos 5 -Wh 20 -Ww 90 -Wl \"$ctitle\" -WL \"$com_icontitle\" -WI \"/MIDAS/MTsort/tcl/bitmaps/sort.icon\" -I"
	    set comprogram "setenv LD_LIBRARY_PATH $libdir;[set bindir]/[set prog] -p$port"
	} {
	    set execcomwindow "exec nohup xterm -fn 8x13 +si -sb -sl 100000 -geometry 90x20+[set xpos]+5 -title \"$ctitle\" -wf -e csh -c "
	    set comprogram "setenv LD_LIBRARY_PATH $libdir;[set bindir]/[set prog] -p$port"
	}
    } {
	set xpos 150
	if {$termtype == "cmdtool"} {
	    set execcomwindow "exec nohup cmdtool -M 10000000 -fn 8x13 -Wp $xpos 5 -Wh 20 -Ww 90 -Wl \"$ctitle\" -WL \"$com_icontitle\" -WI \"/MIDAS/MTsort/tcl/bitmaps/sort.icon\" -I"
	    set comprogram "ssh -t $userid@$host [set bindir]/progstart [set prog]"
	} {
	    set execcomwindow "exec nohup xterm -fn 8x13 +si -sb -sl 100000 -geometry 90x20+[set xpos]+5 -title \"$ctitle\" -wf -e csh -c "
	    set comprogram "ssh -t $userid@$host [set bindir]/progstart [set prog] "
	}
    }	

    set execstring "[set execcomwindow] \"[concat $comprogram]\" &"
    set z [catch {eval $execstring} m]
    if {$z} then {set-footer $m; return }
    
    set execpid $m
    
    write-file $execpidfile $execpid
    
    set i 10
    set z 1
    while {[set z [catch {inform $soap_server readreg-state} m]]} {
	after 1000
	incr i -1
	if {!$i} {break}
    }
    set i 10
    while {[inform $soap_server readreg-state] != "INITIALISEREADY"} {
	after 1000
	incr i -1
	if {!$i} {break}
    }
    
    inform $soap_server refresh-channels
    inform $soap_server refresh-variables
    inform $soap_server refresh-connect
    
    set execloaded 1
}

proc killexecprog {bindir prog} {
    confirm "  TDR program Exit?" "complete-exit $bindir $prog"
    frame-idle
    return
}

proc file_delete_if_exists {f} { if {[file exists $f]} {file delete -force $f} }

proc complete-exit {bindir prog} {
    global execpidfile execpid execloaded host termtype comtitle com_icontitle
    global userid soap_server

    if {!$execpid} {
	set z [catch {set pid [read-file $execpidfile]} m]
	if {!$z} {set execpid $pid}
    }

    set thishost [exec uname -n]

    if {$host == "localhost" || $host == $thishost} {
	puts "Local kill "

	if {$execpid} {
	    set z [catch {set tdrmainpid [read-file /tmp/.pid_$prog]} m]
	    if {$z} {set tdrmainpid ""}
	    if {$tdrmainpid != ""} {catch {exec kill $tdrmainpid}}
	    catch {exec kill $execpid}
	    after 1000
	    
	    if {$soap_server == "greatEB"} {
		file_delete_if_exists /tmp/.pid_TDRinput
		file_delete_if_exists /tmp/.pid_TDRoutput
		file_delete_if_exists /tmp/.pid_TDRcluster
		file_delete_if_exists /tmp/.pid_TDReventsearch
		file_delete_if_exists /tmp/.pid_TDRprefilter
		catch {file_delete_if_exists /tmp/.pid_$prog}
	    } {
		catch {file_delete_if_exists /tmp/.pid_$prog}
	    }

	    set execpid 0
	    file_delete_if_exists $execpidfile
	}
    } {
	puts "Remote kill "
	set xpos 150
	set ctitle "[set comtitle] [set prog]"
	if {$termtype == "cmdtool"} {
	    set execcomwindow "exec nohup cmdtool -M 10000000 -fn 8x13 -Wp $xpos 5 -Wh 20 -Ww 90 -Wl \"$ctitle\" -WL \"$com_icontitle\" -WI \"/MIDAS/MTsort/tcl/bitmaps/sort.icon\" -I"
	    set comprogram "ssh -t $userid@$host [set bindir]/progstop"
	} {
	    set execcomwindow "exec nohup xterm -fn 8x13 +si -sb -sl 100000 -geometry 90x20+[set xpos]+5 -title \"$ctitle\" -wf -e csh -c "
	    set comprogram "ssh -t $userid@$host [set bindir]/progstop"
	}

	set execstring "[set execcomwindow] \"[concat $comprogram]\" &"

        puts "execpid $execpid execstring $execstring"

        if {$execpid != ""} {
        catch {exec kill $execpid}

	set z [catch {eval $execstring} m]
	if {$z} then {set-footer $m}
        }
    
	set execpid 0
        if {$execpidfile != ""} {file_delete_if_exists $execpidfile}
    }

    set execloaded 0
}



