# -----------------------------------------------------------------------------
# File stats.tcl
#
# Source code for EVENT BUILDER WINDOW.
#

set debug 0
set frameOpen 0
set framename .tdrstats
set frametitle "TDR Event Builder"
set icontitle "TDREBStats"

set stats_list ""

if { ![info exists env(TDRBASE)] } {set env(TDRBASE) $env(MIDASBASE)/GREAT/EventBuilder}
set basedir [set env(TDRBASE)]

#set ebfont "10x20"
set ebfont "9x15bold"

set EGreport -1
loadsource common.soap
loadsource common.list
loadsource common.unix

set display_mode 0

proc =display {rows cols } {
    global ebfont
    set fn [self frame-name]
    text $fn.galley -relief sunken -bd 2 -height $rows -width $cols \
	-font $ebfont -bg #bfe7dc -fg black
    pack $fn.galley -side top -pady 5 -padx 10
}

proc make-frame {} {
    global stats_rows frametitle

    :frame [self frame-name] :label $frametitle :show-footer T \
	:icon-name "[self icon-title]" :resize F

    set gapy 15
    :panel P1 :padx 10 :pady 5 :layout T
	set fn [self frame-name]
	label $fn.p1.status -height 2 -width 25 -relief ridge
	place $fn.p1.status -x 50 -y 10
	:button DMODE :label "More Stats" :w 15 :h 2 :x 410 
#	:menubutton BLEN :label "BLOCK LENGTH" :x 330 :y $gapy :strings { 16384 32768 }
#	:message BLEN-VALUE :x 480 :y $gapy
    =display $stats_rows 62
    :panel P3 :padx 10 :pady 5 :layout T
	:button CLEAR :label "CLEAR STATS" :x 10 :y 5 :h 2
	:message CMODE-MSG :label "COUNT MODE" :x 200 :y 0
	:button CMODE :label "" :w 12 :x 200 :y $gapy
	:button UPDATE :label "UPDATE" :x 410 :y 5 :w 15 :h 2
    :show T
}

proc finish args {click-quit}

proc verify-stats0 {} {
    set statslist [first [read-stats]]
    set err 0
    foreach stat $statslist {
	set value [second $stat]
	if {$value} {
	    set err 1
	    break
	}
    }
    return $err
}

proc verify-stats1 {} {
    set err 0
    set statslist [readregs clstats.*]
    foreach stat $statslist {
	set value [second $stat]
	if {$value} {
	    set err 1
	    break
	}
    }
    set statslist [readregs esstats.*]
    foreach stat $statslist {
	set value [second $stat]
	if {$value} {
	    set err 1
	    break
	}
    }
    return $err
}

proc clear-stats {} {
    global display_mode
    clear-stats0
    clear-stats1
}

proc clear-stats0 {} {
    set x [writereg allstats 0]
    after 200
	
    if { [verify-stats0] } {
	set-footer "Failed to clear stats"
    } else {
	set err [catch {exec date} clear_time]
	if {$err} { set clear_time "Nul Nul  0 00:00:00 Nul Nul " }
	set-footer "Stats cleared"
    } 
}

#clear-stats0, allstats actually zeroes these
proc clear-stats1 {} {
    if { [verify-stats1] } {
	set-footer "Failed to clear stats"
    } else {
	set err [catch {exec date} clear_time]
	if {$err} { set clear_time "Nul Nul  0 00:00:00 Nul Nul " }
	set-footer "Stats cleared"
    } 
}

set bgcolour(F)	#eec
set bgcolour(T)	green

proc EVBstatus {} {
   set v [readreg state]
    if { $v == "GOING" } {set r 1} {set r 0}
   return $r
}

proc update-status {} {
    global framename bgcolour soap_server
    set v [readreg state]
    if {$v==-1} { set v "UNKNOWN" }
    if {$v=="?"} { set v "Not Loaded" }
    if { $v=="GOING" } { set t T } {set t F}
    if {![EVBstatus]} {set t F}
    set wid $framename.p1.status
    set err [catch {$wid configure -bg $bgcolour($t) -text "$soap_server: $v"}]
    set p1 [self frame-name].p1
    set p3 [self frame-name].p3
#    set-inactive $t BLEN CLEAR
    return $v
}

set blocklengths { 16384 32768 }

proc update-blen {} {
    set r [readreg outputblocksize]
    :modify BLEN-VALUE :label $r
}

proc click-blen {v} {
    global blocklengths
    update-status
    writereg outputblocksize $v
    update-blen
}

proc click {arg} {
    global debug clickwidgets
    if {$debug}  {puts  "click with argument \{$arg\}"}
    set f [lindex $arg 0]
    set c [lindex $arg 1]
    set w [lindex $arg 2]
    set v [lindex $arg 3]
    set x [lindex $arg 4]
    if {$c==":quit"} then {finish; return}
    if {$f=="ER"} then {remove-eg-error-report self; return}

    foreach wid $clickwidgets {
	set cwid [lindex $wid 0]
	if { [regexp $cwid $w] } {
            set func [second $wid]
	    break
 	}
    }
    if { [info exists func] } {
	eval $func
    } else {
	puts $arg
    }
}

set clickwidgets {
    { CMODE	"click-mode $v"  }
    { BLEN	"click-blen $v"  }
    { UPDATE 	"click-update"   }
    { CLEAR 	"click-clear"    }
    { STATUS	"update-status"  }
    { DMODE     "click-dmode"    }
}

proc click-dmode {} {
    global display_mode
    set display_mode [expr $display_mode +1 & 1]
    click-update
}

proc click-clear {} {
    set-footer "Clearing statistics ..."
    clear-stats
    save-old-counts
    click-update
}

set modes { counts rates }
proc update-cmode {} {
    global mode modes
    set wid [self frame-name].p3.cmode
    set v [$wid cget -text]
    set mode [next-list-value $v $modes]
    $wid configure -text $mode
}

proc click-mode {v} {
    global mode modes 
    update-cmode
}

proc click-quit {} {
    delete-frame
}

proc click-update {} {
    global display_mode
    click-update[set display_mode]
}

proc click-update0 {} {
    global mode 
    :modify [self frame-name] :busy T

    set state [update-status]

    if {$mode == "counts"} {
	save-old-counts
	update-counts0
    } else {
	update-rates0
    }
#    update-blen
    :modify [self frame-name] :busy F
}

proc click-update1 {} {
    global mode 
    :modify [self frame-name] :busy T

    set state [update-status]

    if {$mode == "counts"} {
	save-old-counts
	update-counts1
    } else {
	update-rates1
    }
#    update-blen
    :modify [self frame-name] :busy F
}

proc erase-stats {} {
    global debug
    set wid [self frame-name].galley
    $wid delete 1.0 end
}

set head_fmt "%-14s %28s %14s\n"

set samplePeriod 5000

proc update-rates0 {} {
    global x1 x2 samplePeriod
    set-footer "Updating statistics rates ... wait $samplePeriod msec"
    set x1 [first [read-stats]]
    after $samplePeriod \
	{ set x2 [first [read-stats]]; calculate-rates0  }
}

proc update-rates1 {} {
    global x1 x2 samplePeriod
    set-footer "Updating statistics rates ... wait $samplePeriod msec"
    set x1 [readregs clstats.*]
    append x1 " "
    append x1 [readregs esstats.*]
    after $samplePeriod \
	{ 
	    set x2 [readregs clstats.*]
	    append x2 " "
	    append x2 [readregs esstats.*]
	    calculate-rates1
	}
}

proc calculate-rates0 {} {
    global debug errorCode eb_ldisp head_fmt x1 x2 samplePeriod \
	rates_time rates_time_prev stats_rates stats_rates_prev

    set t $samplePeriod
    set z [catch {exec date} x]
    set stats_rates_prev $stats_rates
    set rates_time_prev ([lindex $rates_time 3])
    if {$z} {
	set err [lindex $errorCode 2]
	set rates_time "Nul Nul  0 00:00:00 Nul Nul "
    } else {
	set rates_time $x
    }
    erase-stats

    set eb_ldisp 1

    put-stats [format $head_fmt\
	"Rates at:" $rates_time $rates_time_prev]

    set nrates [llength $x1]
    set stats_rates {}

    for {set irate 0} {$irate < $nrates} {incr irate} {
	set value1 [lindex $x1 $irate]
	set count1 [second $value1]
	set count2 [second [lindex $x2 $irate]]
	if {$count1 > $count2} {
	    set rate 0;			# Uh, negative rate !
	} else {
	    set rate [expr "($count2 - $count1) / ($t / 1000)"]
	}
	set value_prev [lindex $stats_rates_prev $irate]
	set rate_prev ([second $value_prev])
	set rate_name [first $value1]
	lappend stats_rates [list $rate_name $rate]
	put-stats [format "%-31s %12s %14s" [string range $rate_name 6 end] $rate $rate_prev]
    }
    clear-footer
}

proc calculate-rates1 {} {
    global debug errorCode eb_ldisp head_fmt x1 x2 samplePeriod \
	rates_time rates_time_prev stats_rates stats_rates_prev

    set t $samplePeriod
    set z [catch {exec date} x]
    set stats_rates_prev $stats_rates
    set rates_time_prev ([lindex $rates_time 3])
    if {$z} {
	set err [lindex $errorCode 2]
	set rates_time "Nul Nul  0 00:00:00 Nul Nul "
    } else {
	set rates_time $x
    }
    erase-stats

    set eb_ldisp 1

    put-stats [format $head_fmt\
	"Rates at:" $rates_time $rates_time_prev]

    set nrates [llength $x1]
    set stats_rates {}

    for {set irate 0} {$irate < $nrates} {incr irate} {
	set value1 [lindex $x1 $irate]
	set count1 [second $value1]
	set count2 [second [lindex $x2 $irate]]

	if {$count1 > $count2} {
	    set rate 0;			# Uh, negative rate !
	} else {
	    set rate [expr "($count2 - $count1) / ($t / 1000)"]
	}
	set value_prev [lindex $stats_rates_prev $irate]
	set rate_prev ([second $value_prev])
	set rate_name [first $value1]
	lappend stats_rates [list $rate_name $rate]

	set i [string first "_record" $rate_name]
	if {$i == -1} {continue}
	incr i -1
	set n [string range $rate_name 8 $i]
	
	set prefix [string range $rate_name 0 1]
	set name [readreg [set prefix]name.[set n]]
	put-stats [format "%-14s %-15s %12s %14s"\
		        $n $name $rate $rate_prev]
    }
    clear-footer
}

proc update-counts {} {
    global display_mode
    update-counts[set display_mode]
}

proc update-counts0 {} {
    global stats_values stats_time
    set l [read-stats]
    set val [lindex $l 0]
    if {$val!=-1} {
	set stats_values $val
	set stats_time [lindex $l 1]
	display-counts0
    }
}

proc update-counts1 {} {
    global stats_values stats_time
    set l [read-cles_stats]
    set val [lindex $l 0]
    if {$val!=-1} {
	set stats_values $val
	set stats_time [lindex $l 1]
	display-counts1
    }
}

proc read-stats {} {
    global stats_list
    set t [do-unix date]
    set x [readreg allstats]
    set l ""
    foreach s $stats_list {
	set i [second $s]
	lappend l "[first $s] [lindex $x $i]"
    }
    return [list $l $t]
}

proc read-cles_stats {} {
    set t [do-unix date]
    set x [lsort [readregs clstats.*]]
    append x " "
    append x [lsort [readregs esstats.*]]
    return [list $x $t]
}

proc save-old-counts {} {
    global stats_time stats_time_prev stats_values stats_values_prev

    set stats_values_prev $stats_values
    set stats_time_prev ([lindex $stats_time 3])

}

proc display-counts0 {} {
    global eb_ldisp stats_time stats_time_prev\
        stats_values stats_values_prev head_fmt

    erase-stats
    set eb_ldisp 0
    put-stats [format $head_fmt "Counts at:"\
      $stats_time $stats_time_prev]
    set istat 0

    foreach stat $stats_values {
	set stat_prev [lindex $stats_values_prev $istat]
	incr istat
	set value_prev ([second $stat_prev])
	put-stats [format "%-32s %12s %14s"\
		       [string range [first $stat] 6 end] [second $stat] $value_prev]
    }
    clear-footer
}

proc display-counts1 {} {
    global eb_ldisp stats_time stats_time_prev\
        stats_values stats_values_prev head_fmt

    erase-stats
    set eb_ldisp 0
    put-stats [format $head_fmt "Counts at:"\
      $stats_time $stats_time_prev]
    set istat 0

    foreach stat $stats_values {
	set stat_prev [lindex $stats_values_prev $istat]
	incr istat
	set value_prev ([second $stat_prev])
	set i [string first "_record" [first $stat]]
	if {$i == -1} {continue}
	incr i -1
	set n [string range [first $stat] 8 $i]
	
	set prefix [string range [first $stat] 0 1]
	set name [readreg [set prefix]name.[set n]]
	put-stats [format "%-15s %-16s %12s %14s"\
		        $n $name [second $stat] $value_prev]
    }
    clear-footer
}

proc put-log {s} {
    send sl insert [transparent $s]
}

proc put-stats {s} {
    global fn 
    set wid [self frame-name].galley
    $wid insert end " $s \n"
}

proc init-globals {} {
    global eb_ldisp mode stats_rows rates_time rates_time_prev\
	stats_rates stats_rates_prev stats_time stats_time_prev\
	stats_values stats_values_prev
    global basedir registerlist_file soap_server

    set eb_ldisp -1
    set mode counts
    set rates_time ()
    set rates_time_prev ()
    set stats_rates {}
    set stats_rates_prev {}
    set stats_time ()
    set stats_time_prev ()
    set stats_values_prev {}

    set registerlist_file $basedir/tcl/tdrstats.$soap_server.tcl

    read-statslist

    set stats_rows [llength $stats_values]
    incr stats_rows 3

}

proc read-statslist {} {
    global stats_values stats_list registerlist_file

    set stats_list [read-file $registerlist_file]
    set l [read-stats]
    set val [lindex $l 0]
    if {$val!=-1} {
	set stats_values $val
    }
}

proc define-statslist {} {
    global stats_values stats_list register_list registerlist_file

    set stats_values [readregs stats.*]

    set stats_list ""
    set stats_values [lsort $stats_values]
    foreach s $stats_values {
	set statname [first $s]
	set c $register_list([first $s])
	set i [expr [string range [second [second $c]] 5 end]]
	lappend stats_list "$statname $i"
    }

    write-file $registerlist_file $stats_list
}


set host ""
set port ""

proc tdrstats.host_callback {r} {global host; set host $r}
proc tdrstats.port_callback {r} {global port; set port $r}

proc start {server regfile} {
    global framename frametitle soap_server
    set soap_server $server
    inform $soap_server call-back [set server]stats tdrstats.host_callback readhost
    inform $soap_server call-back [set server]stats tdrstats.port_callback readport
    server-connect $soap_server
    define_registers $regfile
    init-globals
    set frametitle "TDR $server Statistics"
    set framename [string tolower .[set server]stats]
    make-frame
    click-update
    update-cmode
}

