set gain_strings ""
set gains ""
set gain_item 0
set gain_name ""
set gaina_value 0.0
set gainb_value 1.0
set gainc_value 0.0

set gainformat "%-25s %8s %8s %8s" 

proc MakeGainFrame {} {
    global gain_strings gains gain_item gain_name gaina_value gainb_value gainc_value
    global node_name

    set xpos1 12c
    set xpos2 8c
    set xpos3 8c
    set twidth 10

    :frame [self frame-name].setup :label "TDR Gain Matching for $node_name" :show-footer T \
	:icon-name "[self icon-title]" :resize F
    
    :container LEFT :layout H :anchor W :relief R :pady 3

    :panel L1 :layout H :row-gap 10 :col-gap 10
    :layout V

    :list GAINLIST :rows 24 :list-width 370 :x 0 :exclusive T \
	     :double-click F :command click-gainlist :strings $gain_strings :see 0
    [:path GAINLIST].list configure -background white

    :container RIGHT :layout H :anchor E :relief R :pady 3

    set r [first $gain_strings]
    set gain_item 0
    set gain_name [first $r]
    set gaina_value [second $r]
    set gainb_value [third $r]
    set gainc_value [fourth $r]

    :panel L2 :layout H :row-gap 10 :col-gap 10
    :layout H

    :next-row T
    :text GAINAVALUE :label "Modify A:" :value $gaina_value :valuex $xpos1 :w $twidth
    [:path GAINAVALUE].entry configure -background white
    :next-row T
    :text GAINBVALUE :label "Modify B:" :value $gainb_value :valuex $xpos1 :w $twidth
    [:path GAINBVALUE].entry configure -background white
    :next-row T
    :text GAINCVALUE :label "Modify C:" :value $gainc_value :valuex $xpos1 :w $twidth
    [:path GAINCVALUE].entry configure -background white
    :next-row T
    :next-row T
    :next-row T
    :button SAVEGAINS :label "Save current values"
    pack [:path SAVEGAINS] -side right


    pack [:path RIGHT] -after [:path LEFT] -side top

    :show T
}

proc set-gain {} {
    global vxiparamlist gain_strings gains gain_item gain_name
    global gaina_value gainb_value gainc_value

    foreach v $vxiparamlist {
	if {![string compare [fourth $v] $gain_name]} {
	    set label [second $v]
	    set new "$label $gaina_value $gainb_value $gainc_value"
	    set i [lsearch -regexp $gains $label]
	    if {$i == -1} {
		lappend gains $new
	    } {
		set gains [lreplace gains $i $i $new]
	    }
	}
    }
}


proc click-gainavalue {i} {
    global gainformat gain_strings gains gain_item gain_name
    global gaina_value gainb_value gainc_value
    
    puts "click-gainavalue $i"

    set gaina_value $i

    set new "[format $gainformat $gain_name $gaina_value $gainb_value $gainc_value]"
    set gain_strings [lreplace $gain_strings $gain_item $gain_item $new]

    set-gain

    :modify GAINLIST :empty T :strings $gain_strings :see $gain_item
}

proc click-gainbvalue {i} {
    global gainformat gain_strings gains gain_item gain_name
    global gaina_value gainb_value gainc_value
    
    puts "click-gainbvalue $i"

    set gainb_value $i

    set new "[format $gainformat $gain_name $gaina_value $gainb_value $gainc_value]"
    set gain_strings [lreplace $gain_strings $gain_item $gain_item $new]

    set-gain

    :modify GAINLIST :empty T :strings $gain_strings :see $gain_item
}

proc click-gaincvalue {i} {
    global gainformat gain_strings gains gain_item gain_name
    global gaina_value gainb_value gainc_value
    
    puts "click-gaincvalue $i"

    set gainc_value $i

    set new "[format $gainformat $gain_name $gaina_value $gainb_value $gainc_value]"
    set gain_strings [lreplace $gain_strings $gain_item $gain_item $new]

    set-gain

    :modify GAINLIST :empty T :strings $gain_strings :see $gain_item
}

proc click-gainlist {c f i} {
    global gain_strings gains gain_item gain_name gaina_value gainb_value gainc_value
    
    puts "click-gainlist $c... $f... $i"

    set gain_item $i
    set gain_name [first $f]
    set gaina_value [second $f]
    set gainb_value [third $f]
    set gainc_value [fourth $f]

    :modify GAINAVALUE :value $gaina_value
    :modify GAINBVALUE :value $gainb_value
    :modify GAINCVALUE :value $gainc_value
}

proc init-gain-strings {} {
    global vxiparamlist node_name gainformat gains gain_strings

    set gain_strings ""

    set n [string length $node_name]
    set a 0.0
    set b 1.0
    set c 0.0

    foreach v $vxiparamlist {
	if {![string compare -length $n [fourth $v] $node_name]} {
	    #puts "init-gain-strings: $v...$n...$node_name"
	    set label [second $v]
	    set i [lsearch -regexp $gains $label]
	    if {$i == -1} {
		set new "[format $gainformat [fourth $v] 0.0 1.0 0.0]"
		lappend gain_strings "$new"
	    } {
		set r [lindex $gains $i]
		set new "[format $gainformat [fourth $v] [second $r] [third $r] [fourth $r]]"
		lappend gain_strings "$new"
	    }
	}
    }
}

proc read-gain-file {} {
    global basedir usersetupdir node_name 
    global gain_strings gains

    set pf [cluster-type-from-name $node_name]

    set gains ""

    set fn $usersetupdir/gains
    if {![file exists $fn]} {
	return
    }

    puts "Reading defined gains from $fn"

    set gains [read-files $fn]

    init-gain-strings

    return 0
}

proc click-savegains {} {
    global usersetupdir node_name gains

    set pf [cluster-type-from-name $node_name]
    set fn $usersetupdir/gains

    puts "Writing position map to $fn"

    set newlist [lsort -unique -index 0 $gains]

    write-file $fn $newlist
}

proc draw-gain-frame {name} {
    global node_name pos_mode
    set node_name $name
    set z [read-gain-file]
    MakeGainFrame
}




