
set pos1param_strings ""
set pos1_value 0
set pos1_item 0
set pos1_name ""

set pos2params(1) "parameter X 3"
set pos2params(x) "0.0 1.0 0.0"
set pos2params(2) "parameter Y 3"
set pos2params(y) "0.0 1.0 0.0"
set pos_mode 1
set node_name ""

proc MakePos1Frame {} {
    global pos1param_strings pos1params pos1_value
    global node_name

    set xpos1 12c
    set xpos2 8c
    set xpos3 8c
    set twidth 10

    :frame [self frame-name].setup :label "TDR Position Matching Setup for $node_name" :show-footer T \
	:icon-name "[self icon-title]" :resize F
    
    :container LEFT :layout H :anchor W :relief R :pady 3

    :panel L1 :layout H :row-gap 10 :col-gap 10
    :layout V

    :list POSNLIST :rows 24 :list-width 170 :x 0 :exclusive T \
	     :double-click F :command click-posnlist :strings $pos1param_strings :see 0
    [:path POSNLIST].list configure -background white

    :container RIGHT :layout H :anchor E :relief R :pady 3

    :panel L2 :layout H :row-gap 10 :col-gap 10
    :layout H

    :text POS1VALUE :label "Modify:" :value $pos1_value :valuex $xpos1 :w $twidth
    [:path POS1VALUE].entry configure -background white
    :next-row T
    :button SAVEPOSMAP :label "Save current values"

    :panel L3 :layout H :row-gap 10 :col-gap 10
    :layout H

    set ngroups [set pos1params(groups)]
    for {set group 1} {$group <= $ngroups} {incr group} {
	set grpname [second $pos1params(g$group)]
	set origin $pos1params(g[set group]origin)

	:text POS1ORIGIN[set group] :label "Origin [set grpname]:" :value $origin :valuex $xpos1 :w $twidth
	[:path POS1ORIGIN[set group]].entry configure -background white
      :next-row T
    }

    pack [:path L2] -side bottom -expand no
    pack [:path L3] -side top

    pack [:path RIGHT] -after [:path LEFT] -side top

    :show T
}

proc click-pos1origin {g v} {
    global pos1param_strings pos1params pos1_item pos1_value pos1_name

    set grpname $pos1params(g[set g]name)
    set p [set pos1params(g[set g])]
    set pos1params(g[set g]) "[first $p] [second $p] [third $p] origin=$v"

    puts "click-pos1origin $g $v...[set pos1params(g[set g])] "

}

proc click-pos1origin1 {v} {click-pos1origin 1 $v}
proc click-pos1origin2 {v} {click-pos1origin 2 $v}
proc click-pos1origin3 {v} {click-pos1origin 3 $v}
proc click-pos1origin4 {v} {click-pos1origin 4 $v}
proc click-pos1origin5 {v} {click-pos1origin 5 $v}
proc click-pos1origin6 {v} {click-pos1origin 6 $v}
proc click-pos1origin7 {v} {click-pos1origin 7 $v}
proc click-pos1origin8 {v} {click-pos1origin 8 $v}

proc click-pos1value {i} {
    global pos1param_strings pos1params pos1_item pos1_value pos1_name
    
    puts "click-pos1value $i"

    set pos1_value $i
    set pos1params($pos1_name) $pos1_value

    set new "[format "%-10s %10s" $pos1_name $pos1_value]"
    set pos1param_strings [lreplace $pos1param_strings $pos1_item $pos1_item $new]

    :modify POSNLIST :empty T :strings $pos1param_strings :see $pos1_item
}

proc click-posnlist {c f i} {
    global pos1param_strings pos1params pos1_item pos1_value pos1_name
    
    puts "click-posnlist $c... $f... $i"

    set pos1_item $i
    set pos1_value [second $f]
    set pos1_name [first $f]

    :modify POS1VALUE :value $pos1_value
}

proc MakePos2Frame {} {
    global pos2params
    global node_name

    set xpos1 8c
    set xpos2 8c
    set xpos3 8c
    set twidth 10

    :frame [self frame-name].setup :label "TDR Position Matching Setup for $node_name" :show-footer T \
	:icon-name "[self icon-title]" :resize F
    
    :panel FILEING  :layout T :padx 10 :pady 10 
      :layout H; :row-gap 20; :col-gap 10
    :text XA :label "X A:" :value [first $pos2params(x)] :valuex $xpos1 :w $twidth
      [:path XA].entry configure -background white
      :text XB :label "B:" :value [second $pos2params(x)] :valuex $xpos2 :w $twidth
      [:path XB].entry configure -background white
      :text XC :label "C:" :value [third $pos2params(x)] :valuex $xpos3 :w $twidth
      [:path XC].entry configure -background white
      :next-row T
      :text YA :label "Y A:" :value [first $pos2params(y)] :valuex $xpos1 :w $twidth
      [:path YA].entry configure -background white
      :text YB :label "B:" :value [second $pos2params(y)] :valuex $xpos2 :w $twidth
      [:path YB].entry configure -background white
      :text YC :label "C:" :value [third $pos2params(y)] :valuex $xpos3 :w $twidth
      [:path YC].entry configure -background white

      :next-row T
      :button SAVEPOSMAP :label "Save current values"

    :show T
}

proc read-position-file {} {
    global basedir usersetupdir node_name 
    global pos1param_strings pos1params pos2params pos_mode

    set pf [cluster-type-from-name $node_name]

    set pd $usersetupdir/posmaps
    if {[file exists $pd] && [file exists $pd/$pf]} {
	set fn $pd/$pf
    } {
	set fn $basedir/config/posmaps/$pf
    }

    if {![file exists $fn]} {
	set-footer "No position matching file found for $node_name"
	return 1
    }

    puts "Reading position map from $fn"

    set buf [read-files $fn]

    set pos_mode [second [first $buf]]

    if {$pos_mode == 1} {
	set posparams(mode) [first $buf]
	set pos1param_strings ""
	set group 0
	foreach l $buf {

	    if {[first $l] == "#"} {continue}
	    if {[first $l] == "mode"} {
		set pos1params(mode) $l
		continue
	    }
	    if {[first $l] == "parameter"} {
		incr group
		set pos1params(groups) $group
		set pos1params(p$group) $l
		continue
	    }
	    if {[first $l] == "group"} {
		set pos1params(g$group) $l
		set origin [string range [fourth $l] 7 end]
		set pos1params(g[set group]origin) $origin
		set grpname [second $l]
		set pos1params(g[set group]name) $grpname
		set nitems [third $l]
		puts "Group $grpname"
		continue
	    }

	    set pos1params([set grpname].[first $l]) [second $l]
	    lappend pos1param_strings "[format "%-10s %10s" [set grpname].[first $l] [second $l]]"
	    
	}

    } {
	set pos2params(mode) [first $buf]
	set pos2params(1) [second $buf]
	set pos2params(x) [third $buf]
	set pos2params(2) [fourth $buf]
	set pos2params(y) [fifth $buf]

    }
    return 0
}

proc click-saveposmap {} {
    global usersetupdir node_name pos1params pos2params pos_mode

    set pd $usersetupdir/posmaps
    if {![file exists $pd]} {
	create-dir $pd
    }

    set pf [cluster-type-from-name $node_name]
    set fn $pd/$pf

    puts "Writing position map to $fn"

    set buf ""
    if {$pos_mode == 1} {
	lappend buf [set pos1params(mode)]
	set ngroups [set pos1params(groups)]
	
	for {set group 1} {$group <= $ngroups} {incr group} {
	    lappend buf $pos1params(p$group)
	    lappend buf $pos1params(g$group)
	    set grpname [second $pos1params(g$group)]
	    set nitems [third $pos1params(g$group)]

	    for {set i 0} {$i < $nitems} {incr i} {
		set v $pos1params([set grpname].$i)
		lappend buf "$i $v"
	    }
	}

    } {
	lappend buf [set pos2params(mode)]
	lappend buf [set pos2params(1)]
	lappend buf [set pos2params(x)]
	lappend buf [set pos2params(2)]
	lappend buf [set pos2params(y)]
    }

    write-file $fn $buf
}

proc myclick arg {
   global debug chan
   if {$debug} then {puts stderr "myclick with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$f=="ER"} then {cleanup-frame; return}
   if {$c==":quit"} then {click-quit $f; return}
   switch -glob $w {

       XA            {click-newposn x a}
       XB            {click-newposn x b}
       XC            {click-newposn x c}
       YA            {click-newposn y a}
       YB            {click-newposn y b}
       YC            {click-newposn y c}

      SAVE           {click-saveposn}

      default        {puts stdout $arg}
   }
}

proc click-newposn {v xy p} {
    global pos2params
    puts "newposn $xy $p value $v"
    
    switch $xy {
	x {
	    set a [first $pos2params(x)]
	    set b [second $pos2params(x)]
	    set c [third $pos2params(x)]

    puts "newposn $p $a $b $c $v"

	    switch $p {
		a {set pos2params(x) "$v $b $c"}
		b {set pos2params(x) "$a $v $c"}
		c {set pos2params(x) "$a $b $v"}
		}
	    }
	y {
	    set a [first $pos2params(y)]
	    set b [second $pos2params(y)]
	    set c [third $pos2params(y)]
	    switch $p {
		a {set pos2params(y) "$v $b $c"}
		b {set pos2params(y) "$a $v $c"}
		c {set pos2params(y) "$a $b $v"}
	    }
	}
	default {puts "Invalid axis name $xy"}
    }
    
}

proc click-xa {v} {click-newposn $v x a}
proc click-xb {v} {click-newposn $v x b}
proc click-xc {v} {click-newposn $v x c}
proc click-ya {v} {click-newposn $v y a}
proc click-yb {v} {click-newposn $v y b}
proc click-yc {v} {click-newposn $v y c}

proc draw-posn-frame {name} {
    global node_name pos_mode
    set node_name $name
    set z [read-position-file]
    if {!$z} {
	if {$pos_mode == 1} {MakePos1Frame}
	if {$pos_mode == 2} {MakePos2Frame}
    }
}




