
set frame .sysih
set framename .sysih
set frametitle "TDR Input Handler Control"
set icontitle "InputHandler"

set debug 0
set started 0
set sync 0

set usersetupdir [inform greatFiles set usersetupdir]
set runfilename ""

set inhandler ""
set inhandlers "Online Disc Tape"

set handlerloaded ""
set handlercurrent -1
set handlersource ""
set handlerlist {}
set handlercount 0
set handler ""

set status "Not Loaded"

set tracelevel 0

set deviceselected 0

set runfile ""
set runfileselected -1
set runfilelist {}
set runfilecount 0

set currentvolume ""
set currentfile   ""
set currentstart  0
set currentend    0

set filelist {}

loadsource common.soap

#------------------------------------------------------------------------------

proc fopen {name {attr r} } {
    set err [catch {set f [open $name $attr] } m]
    if {$err} {
	display-midas-error-report self "" $m
    }
    return $f
}

proc isComment? {s} {
    if { $s =="" } {
	set b 1
    } else {
	set b [expr { [string index $s 0]=="#" } ]
    }
    return $b
}

proc read-file {f} {
    set buf ""
    set fp [fopen $f r]
    if {$fp==0} return ""
    while 1 {
	set t [gets $fp]
	if {[eof $fp]} {break}
	if { ! [isComment? $t] } { lappend buf $t }
    }
    close $fp
    return $buf
}

proc write-file {f args} {
    set fp [fopen $f w]
    if {$fp==0} {return 1}
    foreach buf $args {
	foreach line $buf {
	    puts $fp $line
	}
    }
    close $fp
    set-footer "Saved to $f"
    return 0
}

proc last p {
   set n [llength $p]
   incr n -1
   return [lindex $p $n]
   }

proc front p {
   set n [llength $p]
   incr n -1
   return [lreplace $p $n $n]
   }

proc confirm {m action} {
   global debug
   set i [tk_dialog [self frame-name].quitdialog "[self frame-title] Confirmation" $m  "warning" 1 "Cancel" "Confirm"]
   if {$i} then $action
   if {$debug} then {puts stderr "confirm: $m choice $i"}
   }

proc click-handler-go {} {
   frame-busy
   set-footer "Starting handler ..."
   repaint
   frame-idle
   }

proc click-handler-stop {} {
   frame-busy
   set-footer "Stopping handler ..."
   repaint
   frame-idle
   }

proc click-help {} {
    inform greatControl click-help
   }


proc click-resetline {} {
   global runfilelist runfileselected
   global currentvolume currentfile currentstart currentend
   if {![llength $runfilelist]} then {return}
   frame-busy
   set currentstart 0
   set currentend 0
   set l "$runfileselected $currentvolume $currentfile $currentstart $currentend"
   set l [format-runfile $l]
   set runfilelist [lreplace $runfilelist $runfileselected $runfileselected $l]
   paint-runfiles
   frame-idle
   }

proc click-remove {} {
   global runfilelist runfileselected
   frame-busy
   if {![llength $runfilelist]} then {
      display-midas-error-report self "" "Runfile list is empty"
      frame-idle
      return
      }
   set runfilelist [lreplace $runfilelist $runfileselected $runfileselected]
   set runfileselected [expr $runfileselected - 1]
   if { $runfileselected < 1 } then {set runfileselected 0}
   paint-runfiles
   frame-idle
   }


proc click-save {} {
   update-program
   update-runfiles
   frame-idle
}

proc update-program {} {
   global runfilelist
   frame-busy
   set i 0
   set z 0
   set m ""
   writereg runfileindex 0
   foreach x $runfilelist {
       writereg inputrunfiledata $x
       incr i
      }
    writereg inputrunfiledata "$i END END 0 0"
    frame-idle
   }

proc click-runlist {c f} {
   global debug
   global runfile runfilelist runfileselected
   global currentvolume currentfile currentstart currentend
   if {$debug} then {puts stderr "click-list: $c $f"}
   clear-footer

   case $c in {
      :list-select {
         set runfile $f
         set runfileselected [first $runfile]
         set currentvolume [second $runfile]
         set currentfile [third $runfile]
         set currentstart [fourth $runfile]
         set currentend [fifth $runfile]
        }
     :list-deselect {
        set runfile ""
         set runfileselected 0
        }
     }          
   if {$runfileselected < 1} then { set runfileselected 0 }
   paint-runfiles
   }

proc update-runfiles {} {
    global runfilelist usersetupdir soap_server

    foreach line $runfilelist {lappend r $line}
    set usersetupdir [inform greatFiles set usersetupdir]
    write-file $usersetupdir/runinfo_[set soap_server] $r
}

proc click-load-runfiles {} {
    global runfilelist usersetupdir soap_server

    set usersetupdir [inform greatFiles set usersetupdir]
    set r [read-file $usersetupdir/runinfo_[set soap_server]]

    set runfilelist $r
    paint-runfiles
    update-program
}

proc load-handler-source {} {
   global handlerloaded debug

    set hs sysih-[set handlerloaded]

   if {$debug} {puts stderr "Loading source for $hs ..."}
#   catch {destroy .sysih}
   loadsource $hs
   set started 0
   start-handler
   set started 1
}

proc read-runinfo {} {
   global runfilename runfilelist runfileselected inhandler

   set runfilelist {}

    if {$inhandler == 0} {
	set l "0 NET NET 0 0"
	lappend runfilelist [format-runfile $l]
	set runfileselected [expr [llength $runfilelist] - 1]
    } {
	set i 0
	set z 0
	writereg runfileindex 0
	while {$z==0} {
	    set info [readreg inputrunfiledata]
	    if {[first $info] != -1} {lappend runfilelist [format-runfile $info]} {break}
	}
	set runfileselected 0
    }
   paint-runfiles
   }

proc format-runfile {r} {
   scan $r "%d %s %s %d %d" index s1 s2 i1 i2
   set r [format "%3d %6s %-30s %12d %12d" $index $s1 $s2 $i1 $i2]
   return $r
   }

proc click-refresh {} { 
   frame-busy
   clear-footer
   read-runinfo
   paint-runfiles
   repaint
   frame-idle
   }

proc finish {} {DeleteTask}

set regfile ""
set host ""
set port ""

proc sysih.regfile_callback {r} {global regfile; set regfile $r}
proc sysih.host_callback {r} {global host; set host $r}
proc sysih.port_callback {r} {global port; set port $r}

proc start args {
    global started inhandler inhandlers handlercurrent handlerloaded debug 
    global soap_server regfile host port

    set soap_server [first $args]
    inform $soap_server call-back sysih sysih.regfile_callback readregisterfile
    inform $soap_server call-back sysih sysih.host_callback readhost
    inform $soap_server call-back sysih sysih.port_callback readport

puts "sysih: $soap_server $regfile $host:$port"
    define_registers $regfile
    server-connect $soap_server

    set inhandler [second $args] 
    set handlerloaded [lindex $inhandlers $inhandler]

   if {$inhandler != $handlercurrent} {
      load-handler-source
      set handlercurrent $inhandler
   } {
      if {$started} then {
	  click-refresh
	  bring-to-front [self frame-name]
      }
   }
}
