/* Definition of layout of cluster output buffers ... */
/* Consists of header followed by one or more group sections */

#ifndef CLUSTEROUTPUTInc
#define CLUSTEROUTPUTInc

#define PILEUP_VETO_BIT_MASK   0x4000
#define PATTERN_VETO_BIT_MASK  0x8000

#define CLUSTER_SEPARATOR      0xa55aa55a
#define CLUSTER_SEPARATOR_TEMP 0xb44bb44b
#define CLUSTER_SEPARATOR_END  0x96699669

#ifdef PASS_RAWDATA_POINTER

typedef struct instanceoutput {
  int instance;
  RawData *param;
} InstanceOutput;

#define CLUSTEROUTPUT_PARAM_LENGTH 8

#else

typedef struct instanceoutput {
  int instance;
  int padding;
#ifdef IN_CLUSTER_CODE
  unsigned long long param;
#else
  RawData param;
#endif
} InstanceOutput;

#define CLUSTEROUTPUT_PARAM_LENGTH 16

#endif


typedef struct groupoutput {
  short startoffset;
  short endoffset;
  int ninstances;
} GroupOutput;

#define CLUSTEROUTPUT_GROUP_PREAMBLE_LENGTH 8

typedef struct clusteroutput {
  int separator;
  int length;
  TimeStamp64 startTS;
  int status;
  unsigned int xpos,ypos;
  short ngroups;
  short maxgroups;
  GroupOutput group[MAX_GROUPS_PER_CLUSTER];
  InstanceOutput data[MAX_INSTANCES];
} ClusterOutput;

/* Following is ClusterOutput length preceeding param */

#define CLUSTER_BUFFER_PREAMBLE  (32 + (8*MAX_GROUPS_PER_CLUSTER))

/* Cluster output buffer header record ...  */

typedef struct clusterheader {
  int number;
  int nitems;
  TimeStamp64 startTS;
} ClusterHeader;

#define CLUSTER_BUFFER_HEADER  sizeof(ClusterHeader)

#endif

