#!../tree_wish -f
# -*-Tcl-*-
#
# dirtree7 - similar to dirtree6, but adds selection of nodes 
#            with mouse button <1>

set started 0

loadsource util
 
set button3 "Todo"

set addmenustrings  "NewSearch NewNode Histogram1D Histogram2D TAC XYmatch Gate2D Output Condition Filter"
set nodemenustrings "InsertNode DeleteNode AddNode AddHistogram1D AddHistogram2D AddTAC AddXYmatch AddGate2D AddOutput Cut Copy Paste ShowVars"
set nodemenu ""
set actionmenustrings "Cut Copy MoveUp MoveDown"
set actionmenu ""
set rootmenustrings "AddSearch "

set current_node ""
set current_mode "AddNode"

proc setup_canvas_right_click {} {
    global menuoptions
    global canvas nodemenu nodemenustrings actionmenu actionmenustrings

    # Setup node specific opoup menu ...
    
    set nodemenu $canvas.nodemenu
    eval menu $nodemenu $menuoptions
    foreach n $nodemenustrings {
	$nodemenu add command -label $n -command nodemenu-$n 
    }

    set actionmenu $canvas.actionmenu
    eval menu $actionmenu $menuoptions
    foreach n $actionmenustrings {
	$actionmenu add command -label $n -command actionmenu-$n 
    }
}

proc process_canvas_right_click {} {
    global button3
    if {$button3 == "Todo"} {addnewnode "" nodebase; redraw_nodes}
}

proc process_node_right_click {} {
    global button3 canvas nodemenu actionmenu

    set tags [$canvas gettags current]
    if {[lindex $tags 1] == "bitmap"} {
	$canvas itemconfig current -background [$canvas cget -selectbackground] \
	    -tags "[$canvas gettags current] selected" 
    } {
	$canvas select clear
	catch {$canvas select from current 0}
	catch {$canvas select to current [string length [$canvas itemcget current -text]]}
    
    }
    
    set path [GetPath $canvas]
    set node [file tail $path]
    
    # Get coords of item bounding box...
    set bbox [$canvas bbox [lindex $tags 2]]
    # Get coords of window (0,0) ...
    set x0 [$canvas canvasx 0]
    set y0 [$canvas canvasy 0]
    # Calculate offset into window ...
    set x [expr int([third $bbox] - [expr $x0])]
    set y [expr int([fourth $bbox] - [expr $y0])]
    # Add in coords of window (0,0) on screen ,,,
    set xw0 [winfo x .]
    set yw0 [winfo y .]
    
    set x [expr $x + $xw0]
    set y [expr $y + $yw0]

    if {[string range $node 0 5] != "action"} {
	tk_popup $nodemenu $x $y 
    } {
	tk_popup $actionmenu $x $y 
    }

    set button3 Done
}


# set the bindings for the tree canvas

proc SetBindings {canvas tree} {
    # bind mouse button <1> to select the label or bitmap
    $canvas bind text <1> "focus %W; SelectNode $canvas"
#    $canvas bind bitmap <1> "SelectBitmap $canvas"
}


# select the current node's label

proc SelectNode {canvas} {
    $canvas select from current 0
    $canvas select to current [string length [$canvas itemcget current -text]]
    DeSelectBitmap $canvas
}


# de-select all node labels

proc DeSelectNode {canvas} {
    $canvas select clear
}


# highlight the node's bitmap

proc SelectBitmap {canvas} {
    catch {focus {}}
    set path [lindex [$canvas gettags current] 0]
    DeSelectNode $canvas
    DeSelectBitmap $canvas
    $canvas itemconfig current -background [$canvas cget -selectbackground] \
	-tags "[$canvas gettags current] selected" 
    puts "selected bitmap: [GetPath $canvas]"
}


# stop highlighting the node's bitmap

proc DeSelectBitmap {canvas} {
    $canvas itemconfig selected -background [$canvas cget -background]
    $canvas dtag selected
}



# return the pathname (dir) for the item currently selected (bitmap or text)

proc GetPath {canvas} {
    set id [$canvas select item]
    if {"$id" == ""} {
        return [lindex [$canvas gettags selected] 0]
    }
    return [lindex [$canvas gettags $id] 0]
}


# add the given dir to the tree
#
# Args: 
#  canvas  - tree's canvas
#  tree    - the tree
#  parent  - pathname of parent dir 
#  dir     - pathname of new dir being added
#  text    - text for tree node label (last component of dir)

proc AddDir {canvas tree parent dir text} {
    global bitmaps
    set bitmap @$bitmaps/dir.xbm

    switch [first $text] {
	action {
	    global [second $text]
	    set type [set [second $text](type)]
	    set name [set [second $text](name)]
	}
	default   {set type default;set name $text}
    }

    set t [$canvas create text 0 0 -text $name -tags [list $dir text $dir:text]]
    set b [$canvas create bitmap 0 0 -bitmap $bitmap -tags [list $dir bitmap $dir:bitmap]]

    set line [$canvas create line 0 0 0 0 -tag "line"]

    switch $type {
	histogram {
	    $canvas itemconfigure $line -fill red
	    $canvas itemconfigure $b -foreground red
	    $canvas itemconfigure $t -fill red
	} 
	tac {
	    $canvas itemconfigure $line -fill purple
	    $canvas itemconfigure $b -foreground purple
	    $canvas itemconfigure $t -fill purple
	} 
	XYmatch {
	    $canvas itemconfigure $line -fill DarkGreen
	    $canvas itemconfigure $b -foreground DarkGreen
	    $canvas itemconfigure $t -fill DarkGreen
	    }
	gate2d {
	    $canvas itemconfigure $line -fill RoyalBlue
	    $canvas itemconfigure $b -foreground RoyalBlue
	    $canvas itemconfigure $t -fill RoyalBlue
	} 
	output {
	    $canvas itemconfigure $line -fill black
	    $canvas itemconfigure $b -foreground black
	    $canvas itemconfigure $t -fill black
	} 
	default {
	    $canvas itemconfigure $line -fill blue
	    $canvas itemconfigure $b -foreground blue
	    $canvas itemconfigure $t -fill blue
	}
    }

    LayoutNode $canvas $tree $dir
    $tree addlink $parent $dir $line -border 5m
}

# Wish SEGVs unless done this way ...
proc DeleteTask {} {
global tree
destroy $tree
delete_task [Get_My_Name]
}

proc click-loadsave {} {
    global soap_server
#    draw-setup-frame
    inform greatFiles start $soap_server
}

proc click-statistics {} {inform stats start}

proc finish {} {
    global started
    set started 0
    DeleteTask
}

proc loadprog_enable {} { [:path loadprog] configure -state normal }
proc loadprog_disable {} { [:path loadprog] configure -state disabled }
proc install_enable {} { [:path install] configure -state normal }
proc install_disable {} { [:path install] configure -state disabled }

proc click-loadprog {} {
    inform greatControl click-loadprog
}

proc click-killprog {} {
    killexecprog
    install_disable
    loadprog_enable
}

# main 
proc start {server} {
    global soap_server env started frame tree canvas dir frametitle icontitle
    global gottext addmenustrings
    global usersetupdir specbase

    set usersetupdir [inform greatFiles set usersetupdir]
    set specbase [inform greatEB set specbase]

    if {$started} {
       bring-to-front [self frame-name]
	return
    } {
	set soap_server $server
	init_soap $server
    }

:frame "[self frame-name]" :label "[self frame-title]" :show-footer T \
        :icon-image "@$env(TCLHOME)/bitmaps/euroball" \
	:icon-name "[self icon-title]"

# Add in buttons at bottom of window ...

set font -*-Helvetica-Bold-R-Normal--13-120-*-*-*-*-iso8859-1
:container FRAME2
set frame2 [:path FRAME2]

#entry $frame2.entry -width 24 -textvariable gottext

:button loadsave :label "Setup Load/Save" :command click-loadsave
[:path loadsave] configure -font $font
:button clustersetup :label "Cluster Setup" :command click-cluster-setup
[:path clustersetup] configure -font $font
:button paramsetup :label "Params" :command draw-params-frame
[:path paramsetup] configure -font $font
:button print :label "Print" :command click-print-canvas
[:path print] configure -font $font

button $frame2.help -text "Help" -command click-help -font $font

pack $frame2.help -side right -expand no -padx 20
#pack $frame2.entry -side left -expand no -padx 20
pack $frame2 -side bottom -padx 10 -pady 10

# Create canvas tree widgets ... 

:container FRAME1
set frame1 [:path FRAME1]
set canvas [MakeCanvas $frame1 $frame1.c]
set tree [tree $canvas.t -parentdistance 2c]

pack $frame1 -side top -padx 10 -pady 10

bind $canvas <Left> "SelectNext $canvas $tree %K"
bind $canvas <Right> "SelectNext $canvas $tree %K"
bind $canvas <Down> "SelectNext $canvas $tree %K"
bind $canvas <Up> "SelectNext $canvas $tree %K"

# add a binding to open/close a listbox listing files in the selected directory

$canvas bind bitmap <1> "focus %W; SelectBitmap $canvas; ShowFiles $canvas $tree"
$canvas bind bitmap <2> "focus %W; SelectBitmap $canvas; ShowActions $canvas $tree"

setup_canvas_right_click
$canvas bind bitmap <3> {set button3 "Todo";focus %W; SelectBitmap $canvas; process_node_right_click}
$canvas bind text <3> {set button3 "Todo";process_node_right_click}
bind $canvas <3> {process_canvas_right_click;set button3 "Todo"}

# Canvas loaded from a virtual directory structure elsewhere ...

wm title . $frametitle
wm iconname . $icontitle
wm geometry . +100+100
#wm deiconify .
:show T

set started 1
}



