
set framename .tdrfilter
set frame .tdrfilter

set icontitle "TDRFilter"
set title "TDR Pre Filter Setup"
set frametitle $title
set started 0

if { ![info exists env(TDRBASE)] } {set env(TDRBASE) $env(MIDASBASE)/GREAT/EventBuilder}

set soap_server greatFilter
set register_file [set env(TDRBASE)]/config/filter_registers

set basedir [set env(TDRBASE)]
set bindir $basedir/bin_$env(DIRPOSTFIX)
set libdir $basedir/lib_$env(DIRPOSTFIX)

if {[string first $libdir [set env(LD_LIBRARY_PATH)]] == -1} {
    set env(LD_LIBRARY_PATH) $libdir:[set env(LD_LIBRARY_PATH)]
}
if {[string first $bindir [set env(PATH)]] == -1} {
    set env(PATH) $bindir:[set env(PATH)]
}

set state ""
set states(0) STOP
set states(1) GO
set statestring(0) STOPPED
set statestring(1) GOING

set blocksizestrings {16384 32768 65536}
set prefilterbgofoldstrings {1 2 3 4 5}

set preferences(blocksize) 16384
set preferences(specbase) /local/filterspectra
set preferences(prefilter_enable) 1
set preferences(prefilter_delay) -500
set preferences(prefilter_width) 500
set preferences(prefilterbgo_enable) 1
set preferences(prefilterbgo_mode) 0
set preferences(prefilterbgo_pileup_mode) 0
set preferences(prefilterbgo_width) 5
set preferences(prefilter1_foldmode) 1
set preferences(prefilter1_foldwidth) 10
set preferences(prefilter2_foldmode) 1
set preferences(prefilter2_foldwidth) 10
set preferences(zeroongo) 0 
set preferences(OUTtransmit) 1
set preferences(inputport) 10305

set outputchannels "0 1 2 3"
set outputmodes { Null RawData Filtered }

set preferences(inputhandler)   1
foreach c $outputchannels {set preferences(outputhandler$c) 0}
foreach c $outputchannels {set preferences(outputenable$c) 0}
foreach c $outputchannels {set preferences(outputdevice$c) /dev/nst0}

load_preferences
set inputport $preferences(inputport)
set specbase $preferences(specbase)
set blocksize $preferences(blocksize)
set prefilter_enable $preferences(prefilter_enable)
set prefilter_delay $preferences(prefilter_delay)
set prefilter_width $preferences(prefilter_width)
set prefilterbgo_enable $preferences(prefilterbgo_enable)
set prefilterbgo_mode $preferences(prefilterbgo_mode)
set prefilterbgo_pileup_mode $preferences(prefilterbgo_pileup_mode)
set prefilterbgo_width $preferences(prefilterbgo_width)
set prefilter1_foldmode $preferences(prefilter1_foldmode)
set prefilter1_foldwidth $preferences(prefilter1_foldwidth)
set prefilter2_foldmode $preferences(prefilter2_foldmode)
set prefilter2_foldwidth $preferences(prefilter2_foldwidth)
set zeroongo $preferences(zeroongo)
set OUTtransmit $preferences(OUTtransmit)

set inhandler $preferences(inputhandler)
foreach c $outputchannels {set outhandler($c) $preferences(outputhandler$c)}
foreach c $outputchannels {set outenable($c) $preferences(outputenable$c)}
foreach c $outputchannels {set outdevice($c) $preferences(outputdevice$c)}


set triggerparamstrings ""
set vxiparamstrings ""

set maxcards 16

loadsource common.soap
set preferences(server-host) localhost
set preferences(server-port) 15550
load_preferences
set host $preferences(server-host)
set port $preferences(server-port)

loadsource sysio
loadsource greatExec

proc make-frame-diagnostics {} {

    set fn [self frame-name]diag
    set ft "GREAT Event Filter Diagnostics"
    :frame $fn :label $ft :show-footer T \
	:icon-name "Diag" :resize F
    [:path $fn] configure -background wheat

    set xpos1 15c
    set twidth2 19

    :container DIAG :layout T :anchor N :relief R :pady 3
    :panel D1 :layout L :pady 3 :padx 7 
    :layout H
    :choice DEBUG :label "    Level" :strings { 0 1 2 3 4 5 } :w 2 

    for {set j 0} {$j < 6} {incr j} {
	[:path DEBUG].button$j configure -selectcolor pink
    }

    :button DIAGREFRESH :label "Refresh" :w 10

    :show T
}

proc make-frame {} {
    global frametitle OUTtransmit state host port
    global inhandler inhandlers outhandler outhandlers specbase
    global outenable outputlabel outputchannels outputmodes noutputmodes
    global prefilter_enable prefilter_delay prefilter_width
    global prefilterbgo_enable prefilterbgo_mode prefilterbgo_width
    global prefilterbgo_pileup_mode
    global blocksize blocksizestrings
    global prefilter1_foldmode prefilter1_foldwidth prefilterbgofoldstrings
    global prefilter2_foldmode prefilter2_foldwidth

   set xpos 10c
   set xpos1 15c
   set xpos2 15c
   set xpos3 20c
   set xpos4 30c
   set xpos5 15c
   set twidth 30
   set twidth2 19

    :frame [self frame-name] :label $frametitle :show-footer T \
	:icon-name "[self icon-title]" :resize F
    [:path [self frame-name]] configure -background wheat

    :container CONNECT :layout T :anchor N :relief R :pady 3

    :panel IP :layout T :padx 5 :pady 2 :ipadx 5 :ipady 2 :relief R 
    :layout H; :row-gap 0; :col-gap 0
    
    :text HOST :label "Task Host:" :w 10 :value $host 
    :text PORT :label "   Port:" :w 6 :value $port
    [:path HOST].entry configure -background cornsilk
    [:path PORT].entry configure -background cornsilk
    :message CONNECT_STATE :label "Not Connected"

    :text STATE :label "Status:" :value $state \
	:w $twidth2

    pack [:path STATE] -side right

   :container STATECON :layout T :anchor N :relief R :pady 3
    :panel P4 :layout T :pady 3 :padx 2 
    :layout H

    :checkbox OUTTRANSMIT :w 25 :label "Transmit to Output" :value $OUTtransmit
    :checkbox ZEROONGO  :label "Zero Spectra on Go" :value 0 :x 450

    :next-row T

    :checkbox PREFILTERBGO :w 20 :valuex 120 :label "BGOfilter Enable" :value $prefilterbgo_enable :command click-prefilterbgo
    :checkbox FILTERBGOMODE  :label "Veto:" \
	:value $prefilterbgo_mode :command click-prefilterbgomode
    :checkbox FILTERBGOPILEUPMODE  :label "GePileup:" \
	:value $prefilterbgo_pileup_mode :command click-prefilterbgopileupmode
    :text FILTERBGOWIDTH :label "Width:" :value $prefilterbgo_width \
	:x 280 :w 8 :command click-prefilterbgowidth
 
    :choice-stack FILTER1FOLDMODE :label "Fold:" :x 400 \
	:menu $prefilterbgofoldstrings
    [:path FILTER1FOLDMODE].message configure -text $prefilter1_foldmode
    :text FILTER1FOLDWIDTH :label "Width:" :value $prefilter1_foldwidth \
	:w 8 :command click-prefilter1foldwidth
 
    :next-row T

    :checkbox PREFILTER :w 20 :valuex 120 :label "Fastfilter Enable" :value $prefilter_enable :command click-prefilter
    :text FILTERDELAY :w 8 :valuex 80 :label "Delay:" \
	:value $prefilter_delay :command click-prefilterdelay
    :text FILTERWIDTH :label "Width:" :value $prefilter_width \
	:x 280 :w 8 :command click-prefilterwidth
 
 
    :choice-stack FILTER2FOLDMODE :label "Fold:" :x 400 \
	:menu $prefilterbgofoldstrings
    [:path FILTER2FOLDMODE].message configure -text $prefilter2_foldmode
    :text FILTER2FOLDWIDTH :label "Width:" :value $prefilter2_foldwidth \
	:w 8 :command click-prefilter2foldwidth

    :container ACQ  :layout T :anchor N :relief R :pady 3
    :panel P5 :layout T :pady 3 :padx 2 
    :layout H

#    :choice GOSTOP :label "Acquisition" :strings { Stop Go } :valuex $xpos2 :w 4
#    [:path GOSTOP].button0 configure -selectcolor red
#    [:path GOSTOP].button1 configure -selectcolor green

    :choice-stack BLOCKSIZE :label "BlockSize:" :x 450  \
	:menu $blocksizestrings 
    [:path BLOCKSIZE].message configure -text $blocksize

    :next-row T
    :text SPECTRUMDIR :label "Spectrum path:" :value $specbase :valuex $xpos1 :w 39
    [:path SPECTRUMDIR].entry configure -background cornsilk
    :button SAVESPECTRA :label "Save Shm Spectra to disc" :x 440


    :container IN :layout T :anchor N :pady 3
    :panel INCONTROL :layout T :anchor NW :pady 3
    :layout H; :row-gap 2; :col-gap 10
    :choice-stack INHANDLER :label "Input" :menu $inhandlers :valuex $xpos
    [:path INHANDLER].message configure -width 100
    :button INRUNFILES :label "RunFiles Setup" :w 15 :x 180

    foreach oc $outputchannels {
	:container OUT$oc :layout T :anchor N :pady 3
	:panel OUTCONTROL-$oc :layout T :anchor NW :pady 3
	:layout H; :row-gap 2; :col-gap 10
	:choice-stack OUTHANDLER-$oc :label "Output $oc" :menu $outhandlers :valuex $xpos
	:button OUTRUNFILES-$oc :label "RunFiles Setup" :w 15 :x 180

	:choice OUTMODES-$oc :label "Enable" :strings $outputmodes :w 8
	set noutputmodes [llength $outputmodes]
	for {set j 0} {$j < $noutputmodes} {incr j} {
	    [:path OUTMODES-$oc].button$j configure -selectcolor pink
	}

    }

    :container OUT :layout T :anchor N :pady 3

    :panel BUTTONS
     :layout H; :col-gap 5
    :button STATISTICS :label "Statistics" :w 7 :x 10

    :button DISPLAYSPECTRA :label "Display spectra" :w 12
    :button DIAGNOSTICS :label "Diagnostics" :w 10
    :button REFRESH :label "Refresh" :w 8
    :button MYHELP :label "Help" :w 4
    
    pack [:path MYHELP] -side right
    pack [:path REFRESH] -side right
    pack [:path DIAGNOSTICS] -side right
    pack [:path DISPLAYSPECTRA] -side right

    :show T
#   set z [catch {wm geometry [self frame-name] -10-10} m]
#   if {$z} {set z [catch {wm geometry . -10-10} m]}
   update
}

proc set-browsemenu {m} {
    global _winname
    global triggerparamstrings
    set mwid [:path $m].menu
    $mwid delete 0 end
    set ii 0
    set name [lowercase $m]
    set win $_winname($name)

    foreach c $triggerparamstrings {
	set d [first $c]
	$mwid add cascade -label "$d" -menu $mwid.c$ii
	catch {menu $mwid.c$ii -tearoff false}
	:_set-font $mwid.c$ii menu 0 N
	$mwid.c$ii delete 0 end
	set jj 0
	
	foreach ss $c {
	    if {$jj > 0} {
		if {[expr $jj % 8] == 0} {set col 1} {set col 0}
		$mwid.c$ii add command -label "$ss" -columnbreak $col \
		    -command ":mymenubutton-event $win $name $d $ss"
	    }
	    incr jj
	}
	incr ii
    }
}

proc :mymenubutton-event {win name crate chan} {
    global frame _commandname

    set i [string first help $name 0]
    set wid [string toupper [string replace $name $i [expr $i + 4] -]]
    set x $crate.$chan
    :modify $wid :value $x
    
#    click-addresshelp $name $crate $chan
}


proc xfopen {name {attr r} } {
    set err [catch {set f [open $name $attr] } m]
    if {$err} {
	display-error-report self $m
	set f 0
    }
    return $f
}

proc xisComment? {s} {
    if { $s =="" } {
	set b 1
    } else {
	set b [expr { [string index $s 0]=="#" } ]
    }
    return $b
}

proc xread-file {f} {
    set buf ""
    set fp [fopen $f r]
    if {$fp==0} return ""
    while 1 {
	set t [gets $fp]
	if {[eof $fp]} {break}
	if { ! [isComment? $t] } { lappend buf $t }
    }
    close $fp
    return $buf
}

proc refresh-variables {} {
    global prefilterbgo_enable
    global prefilterbgo_mode
    global prefilterbgo_pileup_mode
    global prefilterbgo_width
    global prefilter1_foldmode
    global prefilter1_foldwidth
    global prefilter2_foldmode
    global prefilter2_foldwidth
    global prefilter_enable
    global prefilter_width prefilter_delay
    global OUTtransmit

    writereg transmit.output $OUTtransmit

    writereg prefilter1 $prefilterbgo_enable
    writereg prefilter1mode $prefilterbgo_mode
    writereg prefilter1pileupmode $prefilterbgo_pileup_mode
    writereg prefilter1width $prefilterbgo_width
    writereg prefilter1foldmode $prefilter1_foldmode
    writereg prefilter1foldwidth $prefilter1_foldwidth
    writereg prefilter2foldmode $prefilter2_foldmode
    writereg prefilter2foldwidth $prefilter2_foldwidth
    writereg prefilter $prefilter_enable
    writereg prefilterdelay $prefilter_delay
    writereg prefilterwidth $prefilter_width
}

proc read-state {} {
    return [readreg state]
}

proc write-state {v} {
    global states
    set z [catch {writereg state [set states($v)]} m]
    if {$z} {set-footer $m} {clear-footer}
    return $z
}

proc refresh {} {
    global OUTtransmit
    global prefilter_enable prefilter_width prefilter_delay
    global prefilterbgo_enable prefilterbgo_width prefilterbgo_mode
    global  prefilterbgo_pileup_mode
    global prefilter1_foldmode prefilter1_foldwidth
    global prefilter2_foldmode prefilter2_foldwidth
    global state blocksize

    set v [readreg state]
    if {$v == "?"} {return}
    :modify STATE :value [readreg state]
    :modify ZEROONGO :value [readreg zerospectraongo]

    set v [readreg transmit.output]
    if {$v != "?"} {set OUTtransmit $v}
    :modify OUTTRANSMIT :value $v

    set v [readreg outputblocksize]
    if {$v != "?"} {set blocksize $v}
    [:path BLOCKSIZE].message configure -text $blocksize

#    set v [readreg prefilter1]
#    if {$v != "?"} {set prefilterbgo_enable $v}
#    :modify PREFILTERBGO :value $prefilterbgo_enable

#    set v [readreg prefilter1width]
#    if {$v != "?"} {set prefilterbgo_width $v}
#    :modify FILTERBGOWIDTH :value $prefilterbgo_width

#    set v [readreg prefilter1mode]
#    if {$v != "?"} {set prefilterbgo_mode $v}
#    :modify FILTERBGOMODE :value $prefilterbgo_mode

#    set v [readreg prefilter1pileupmode]
#    if {$v != "?"} {set prefilterbgo_pileup_mode $v}
#    :modify FILTERBGOPILEUPMODE :value $prefilterbgo_pileup_mode

#    set v [readreg prefilter1foldwidth]
#    if {$v != "?"} {set prefilter1_foldwidth $v}
#    :modify FILTER1FOLDWIDTH :value $prefilter1_foldwidth

#    set prefilter1_foldmode [readreg prefilter1foldmode]
#    if {$v != "?"} {if {!$prefilter1_foldmode} {set prefilter1_foldmode 1}}
#    :modify FILTER1FOLDMODE :value [expr $prefilter1_foldmode -1]

    set v [readreg prefilter2foldwidth]
    if {$v != "?"} {set prefilter2_foldwidth $v}
    :modify FILTER2FOLDWIDTH :value $prefilter2_foldwidth

    set prefilter2_foldmode [readreg prefilter2foldmode]
    if {$v != "?"} {if {!$prefilter2_foldmode} {set prefilter2_foldmode 1}}
    :modify FILTER2FOLDMODE :value [expr $prefilter2_foldmode -1]

    set v [readreg prefilter]
    if {$v != "?"} {set prefilter_enable $v}
    :modify PREFILTER :value $prefilter_enable

    set v [readreg prefilterwidth]
    if {$v != "?"} {set prefilter_width $v}
    :modify FILTERWIDTH :value $prefilter_width

    set v [readreg prefilterdelay]
    if {$v != "?"} {set prefilter_delay $v}
    :modify FILTERDELAY :value $prefilter_delay

}

proc get-vxiparam-list {} {
    global vxiparamlist vxiparamstrings
    global maxcards

    if {!$maxcards} {
      set configdir [inform greatControl set configdir]
      set buf [read-file $configdir/VXIconfig]
    } {
      set buf ""
      set label 0
      for {set card 0} {$card <= $maxcards} {incr card} {
         for {set chan 0} {$chan < 32} {incr chan} {
            lappend buf "VXI$card.c$chan $card $card $chan $label"	
            incr label
         }
      }
    }
    
    set vxiparamlist ""
    set vxiparamstrings ""
    set prevcrate ""
    set cratelist ""
    foreach l $buf {
	if {$l == ""} {continue}
	set name [first $l]
	set label [fifth $l]
	
	lappend vxiparamlist "$name $label "
	
	# Extract crate name and chan name ...
	set n [string first "." $name]
	incr n -1
	set crate [string range $name 0 $n]
	incr n 2
	set chan [string range $name $n end]
	
	if {$prevcrate == ""} {
	    set prevcrate $crate
	    append cratelist "$crate "
	}
	
	if {$prevcrate == $crate} {
	    append cratelist "$chan "
	} {
	    lappend vxiparamstrings $cratelist
	    set cratelist "$crate $chan "
	    set prevcrate $crate
	}	
	
    }
    lappend vxiparamstrings $cratelist
}

proc get-param-list {} {
    send greatTree call-back greatFilter greatFilter.get-param-list-continue get-param-list-all
}

proc greatFilter.get-param-list-continue {l} {
    global triggerparamstrings triggerparamlist
    set triggerparamlist $l
    # Generate full names ...
    set clusters ""
    set cluster ""
    set prev ""
    foreach d $l {
	set name [first $d]
	set nn [split $name "_"]
	if {[first $nn] == $name} {
	    set nn [split $name "."]
	} {
	    set nn [split $name "_"]
	}
	set cl [first $nn]
	if {$prev == ""} {set prev $cl}

	if {$prev != $cl} {
	    lappend clusters "$prev $cluster"
	    set cluster ""
	    set prev $cl
	}
	append cluster "[second $nn] "
    }
    lappend clusters "$cl $cluster"
    set triggerparamstrings $clusters
}

proc click-state {v} {
}

proc create-dir {newdir} {
   global errorCode
	puts "Creating $newdir..."
   set z [catch {exec mkdir $newdir} m]
   if {$z!=0 && $errorCode!="NONE"} then {
      display-midas-error-report self "" \
              "Attempting to create directory $newdir:\n" $m
      error $m
      }
   }

proc spectrum-directory-ok {} {
    global specbase
   if {![file exists $specbase]} {
       set i [string last "/" $specbase]
       if {$i == -1} {
	   set-footer "Invalid spectrum directory name!"
	   return 0
       }
       set sup [string range $specbase 0 [incr i -1]]
       if {![file exists $sup]} {
	   set-footer "Spectrum directory $sup does not exist!"
	   return 0
       } {
	   create-dir $specbase
       }
   }

    if {![file writable $specbase]} {
	   set-footer "Spectrum directory not writable!"
	   return 0
    } {
	clear-footer
    }
    return 1
}

proc click-spectrumdir {f} {
    global specbase preferences
    clear-footer
    set specbase $f
    writereg specdir $f
    
    set preferences(specbase) $specbase
    update_preferences
    
    click-refresh
   }

proc click-savespectra {} {
   global debug specbase bindir
   if {$debug} then {puts stderr "click-save-spectra to $specbase"}
   frame-busy
   clear-footer
   if {$specbase == ""} {set specbase $env(HOME)}
   set z [catch {exec $bindir/TDRshmsave -sfilter -p$specbase} m]
   if {$z!=0} then {
      puts stderr "Spectrum save error ..."
      puts stderr $m
      }
   frame-idle
   }

proc click-gostop {v} {
    global states statestring
    set s [readreg state]
    if {$s == "?"} {set-footer "Program not loaded"; return}
    set z [catch {writereg state [set states($v)]} m]
    if {$z} {set-footer $m} {clear-footer}

    set i 10
    while {$i} {
	after 500
	set s [readreg state]
	if {$s == "?"} {break}
	if {$s == [set statestring($v)]} {break}
	incr i -1
    }
    if {$s != [set statestring($v)]} {
	set-footer "Program in wrong state for [set states($v)]"
    }
    :modify STATE :value $s
#    if {$s == "GOING"} {set v 1} {set v 0}
#    :modify GOSTOP :value $v
}

proc click-zeroongo {v} {
    global zeroongo preferences
    set zeroongo $v
    writereg zerospectraongo $v
    set preferences(zeroongo) $v
    update_preferences
}

proc click-OUTtransmit {v} {
    global OUTtransmit preferences
    set OUTtransmit $v
    writereg transmit.output $v
    set preferences(OUTtransmit) $v
    update_preferences
}

proc click-refresh-diagnostics {} {
    set v [readreg debug]
    :modify DEBUG :value $v 
}

proc refresh-connect {} {
    global soap_server execloaded
    set execloaded [server_ping $soap_server]
    if {!$execloaded} {
	set-footer "Program not loaded!"
	inform greatControl log-put \"Filter Setup Program not loaded\"
	:modify CONNECT_STATE :label "Not Connected"
	return
    }
    :modify CONNECT_STATE :label "Connected"
}

proc click-refresh {} {

    refresh-connect

    set s [readreg state]
#    if {$s == "GOING"} {set v 1} {set v 0}
#    :modify GOSTOP :value $v

    click-refresh-channels
    refresh
}

proc startup-refresh {} {

    refresh-connect

    set s [readreg state]
    if {$s == "?"} {return}
    :modify STATE :value $s
#    if {$s == "GOING"} {set v 1} {set v 0}
#    :modify GOSTOP :value $v

    click-refresh-channels
    refresh-variables
}

proc click-statistics {} {
    global soap_server register_file
    inform [set soap_server]stats start $soap_server $register_file
}

proc click-diagnostics {} {
  make-frame-diagnostics
}

proc click-prefilterbgo {w v} {
    global prefilterbgo_enable preferences
    set prefilterbgo_enable $v
    writereg prefilter1 $prefilterbgo_enable
    set preferences(prefilterbgo_enable) $v
    update_preferences
}

proc click-prefilterbgomode {w v} {
    global prefilterbgo_mode preferences
    set prefilterbgo_mode $v
    writereg prefilter1mode $prefilterbgo_mode
    set preferences(prefilterbgo_mode) $v
    update_preferences
}

proc click-prefilterbgopileupmode {w v} {
    global prefilterbgo_pileup_mode preferences
    set prefilterbgo_pileup_mode $v
    writereg prefilter1pileupmode $prefilterbgo_pileup_mode
    set preferences(prefilterbgo_pileup_mode) $v
    update_preferences
}

proc click-prefilterbgowidth {w v} {
    global prefilterbgo_width preferences
    set prefilterbgo_width $v
    writereg prefilter1width $prefilterbgo_width
    set preferences(prefilterbgo_width) $v
    update_preferences
}

proc click-prefilter1foldmode {v} {
    global prefilter1_foldmode preferences
    set prefilter1_foldmode $v
    incr prefilter1_foldmode
    writereg prefilter1foldmode $prefilter1_foldmode
    set preferences(prefilter1_foldmode) $v
    update_preferences
}

proc click-prefilter1foldwidth {w v} {
    global prefilter1_foldwidth preferences
    set prefilter1_foldwidth $v
    writereg prefilter1foldwidth $prefilter1_foldwidth
    set preferences(prefilter1_foldwidth) $v
    update_preferences
}

proc click-prefilter2foldmode {v} {
    global prefilter2_foldmode preferences
    set prefilter2_foldmode $v
    incr prefilter2_foldmode
    writereg prefilter2foldmode $prefilter2_foldmode
    set preferences(prefilter2_foldmode) $v
    update_preferences
}

proc click-prefilter2foldwidth {w v} {
    global prefilter2_foldwidth preferences
    set prefilter2_foldwidth $v
    writereg prefilter2foldwidth $prefilter2_foldwidth
    set preferences(prefilter2_foldwidth) $v
    update_preferences
}

proc click-prefilter {w v} {
    global prefilter_enable preferences
    set prefilter_enable $v
    writereg prefilter $prefilter_enable
    set preferences(prefilter_enable) $v
    update_preferences
}

proc click-prefilterdelay {w v} {
    global prefilter_delay preferences
    set prefilter_delay $v
    writereg prefilterdelay $prefilter_delay
    set preferences(prefilter_delay) $v
    update_preferences
}

proc click-prefilterwidth {w v} {
    global prefilter_width preferences
    set prefilter_width $v
    writereg prefilterwidth $prefilter_width
    set preferences(prefilter_width) $v
    update_preferences
}

proc click-blocksize {v} {
    global blocksize blocksizestrings preferences
    set blocksize [lindex $blocksizestrings $v]
    writereg outputblocksize $blocksize
    set preferences(blocksize) $blocksize
    update_preferences

    [:path BLOCKSIZE].message configure -text $blocksize
}

proc click-install {} {
    global env started usersetupdir soap_server host specbase
    global clustersetupname nodesetupname execloaded

    set execloaded [server_ping $soap_server]
    if {!$execloaded} {
	set-footer "Program not loaded!"
	inform greatControl log-put \"Filter Setup Program not loaded\"
	return
    }

    set s [readreg state]
    if {$s == "GOING"} {
	set-footer "Not whilst going!"
	inform greatControl log-put \"Filter Setup ... Not whilst going!\"
	return
    }

    writereg specdir $specbase

    inform greatControl log-put \"Filter Setup started ... \"
    set-footer "Please wait ... setup in progress"

    set usersetupdir [inform greatFiles set usersetupdir]

    set localsetupdir /local/TDRfilterSetup

# If remote then copy setup files over ...

    set thishost [exec uname -n]
    if {$host != "localhost" && $host != $thishost} {
	set z [catch {writereg setupdir [set localsetupdir]} result]
	if {$z} {
	    set i [expr [string first "\n" $result] -1]
	    if {$i < 0} {set out end} {set out $i}
	    set-footer "Setup initialisation failed: [string range $result 0 $out]"
	    return
	}
	catch {file mkdir /net/[set host][set localsetupdir]}
	file copy -force [set usersetupdir]/currentsetup.cdef /net/[set host][set localsetupdir]
	file copy -force [set usersetupdir]/currentsetup.ndef /net/[set host][set localsetupdir]
    } {
	set z [catch {writereg setupdir $usersetupdir} result]
	if {$z} {
	    set i [expr [string first "\n" $result] -1]
	    if {$i < 0} {set out end} {set out $i}
	    set-footer "Setup initialisation failed: [string range $result 0 $out]"
	    return
	}
    }

    set exptdir [set env(EXPERIMENTDIR)]
    set z [catch {writereg experiment $exptdir} result]

    set z [catch {writereg state INITIALISE} result]
   
    if {$z} {
	set i [expr [string first "\n" $result] -1]
	if {$i < 0} {set out end} {set out $i}
	set-footer "Setup initialisation failed: [string range $result 0 $out]"
	return
    }

    set n 0
    while {$n < 100} {
       after 200
       set s [readreg state]
       if {$s == "INITIALISED"} {break}
       if {$s == "UNINITIALISED"} {break}
       if {$s == -1} {break}
       incr n
       }

    if {$s == "INITIALISED"} {
	set t [clock format [clock seconds]]
	inform greatControl log-put "\"Filter Setup installed ... $t\""
	inform greatControl set-footer "Filter Setup installation completed"
	set-footer "Setup installation completed"
    }

    if {$s == "UNINITIALISED"} {
	set t [clock format [clock seconds]]
	inform greatControl log-put "\"Filter Setup install failed ... $t\""
	inform greatControl set-footer "Filter Setup installation failed"
	set-footer "Setup installation failed"
    }

     click-refresh
}

proc re-refresh {} {click-refresh; after 1000 re-refresh}

proc click-debug {v} {writereg debug $v}

proc click-display-spectra {} {
   global debug
   if {$debug} {puts stderr "click-display-spectra"}
   frame-busy
   clear-footer
   inform sb sortstart 1
   frame-idle
   }

proc click {arg} {
    global debug clickwidgets
    if {$debug}  {puts  "click with argument \{$arg\}"}
    set f [lindex $arg 0]
    set c [lindex $arg 1]
    set w [lindex $arg 2]
    set v [lindex $arg 3]
    set x [lindex $arg 4]
    if {$c==":quit"} then {finish; return}
    if {$f=="ER"} then {remove-eg-error-report self; return}

    foreach wid $clickwidgets {
	set cwid [lindex $wid 0]
	if { [regexp $cwid $w] } {
            set func [second $wid]
	    break
 	}
    }
    if { [info exists func] } {
	eval $func
    } else {
	puts $arg
    }
}

set clickwidgets {
    { STATE             "click-state $v" }
    { HOST              "click-host $v;refresh-connect" }
    { PORT              "click-port $v;refresh-connect" }
    { GOSTOP            "click-gostop $v" }
    { ZEROONGO          "click-zeroongo $v" }
    { OUTTRANSMIT       "click-OUTtransmit $v" }
    { BLOCKSIZE         "click-blocksize $v" }
    { STATISTICS        "click-statistics" }

    { SPECTRUMDIR       "click-spectrumdir $v" }
    { SAVESPECTRA       "click-savespectra" }

    { INHANDLER         "click-inhandler $v" }
    { INRUNFILES        "click-inrunfiles" }

    { OUTHANDLER*       "click-outhandler $v [string range $w end end]" }
    { OUTRUNFILES*      "click-outrunfiles [string range $w end end]" }
    { OUTMODES*         "click-outenable $v [string range $w end end]" }

    { PREFILTERBGO      "click-prefilterbgo $v"}
    { FILTERBGOMODE     "click-prefilterbgomode $v"}
    { FILTERBGOPILEUPMODE     "click-prefilterbgopileupmode $v"}
    { FILTERBGOWIDTH    "click-prefilterbgowidth $v"}
    { FILTER1FOLDMODE   "click-prefilter1foldmode $v"}
    { FILTER1FOLDWIDTH  "click-prefilter1foldwidth $v"}

    { PREFILTER         "click-prefilter $v"}
    { FILTERDELAY       "click-prefilterdelay $v"}
    { FILTERWIDTH       "click-prefilterwidth $v"}
    { FILTER2FOLDMODE   "click-prefilter2foldmode $v"}
    { FILTER2FOLDWIDTH  "click-prefilter2foldwidth $v"}

    { DEBUG             "click-debug $v" }
    { DIAGNOSTICS       "click-diagnostics" }
    { DIAGREFRESH       "click-refresh-diagnostics" }
    { DISPLAYSPECTRA    "click-display-spectra" }
    { REFRESH           "click-refresh" }
}

proc readreg-state {} {return  [readreg state]}
proc writereg-state {v} {return  [writereg state $v]}

proc readregisterfile {} {global register_file; return $register_file}
proc readhost {} {global host; return $host}
proc readport {} {global port; return $port}

proc readinputport {} {global inputport; return $inputport}
proc writeinputport {p} {
    global inputport preferences
    set inputport $p
    set preferences(inputport) $inputport
    update_preferences
}

proc call-back {interp proc result} {
   global debug
   if {$debug} then {puts stderr "[self frame-name]: call-back: $interp $proc $result"}
   send $interp $proc [transparent [$result]]
   }

proc start args {
    global started soap_server register_file
    if {$started} {
	bring-to-front [self frame-name]
    } {
	define_registers $register_file
	set there [server-connect $soap_server]
	get-param-list
	get-vxiparam-list
	make-frame
	set started 1
    }
    startup-refresh
}
