
set frametitle "TDR Output Tape Handler Control"

set z [catch {set env(OSTYPE)} m]
if {$z} {set ostype solaris} {set ostype $m}
if {$ostype == "solaris"} {
  set devlist {"Not_Active" "/dev/rmt/0mbn" "/dev/rmt/1mbn" "/dev/rmt/2mbn" "/dev/rmt/0hbn" "/dev/rmt/1hbn" "/dev/rmt/2hbn" "/dev/rmt/0cbn" "/dev/rmt/1lbn" "Other?"}
  } {
  set devlist {"Not_Active" "/dev/nst0" "/dev/nst1" "/dev/nst2" "Other?"}
  }


#------------------------------------------------------------------------------

proc click arg {
   global debug chan
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$f=="ER"} then {cleanup-frame; return}
   if {$c==":quit"} then {click-quit; return}
   clear-footer
   switch -glob $w {

      ADD                    {click-add}
      INSERT                 {click-insert}
      REPLACE                {click-replace}
      REMOVE                 {click-remove}
      RESETFILE              {click-resetline}
      SAVE                   {click-save}

      MYHELP                 {click-help}
      REFRESH                {click-refresh}

      RUNLIST                {click-runlist $c $v}

      READRUNFILES           {click-load-runfiles}

      VOLUME                 {click-volume $v}
      FILE                   {click-file $v}
      STARTBLOCK             {click-startblock $v}
      ENDBLOCK               {click-endblock $v}

      OUTPUT-HANDLER          {click-output-handler $v}
      DEVICE                 {click-device $v}

      default                {puts stdout $arg}
   }
}
#------------------------------------------------------------------------------

proc make-frame {} {
   global debug env outputlabel outchannel
   global handlerlist handlerselected handlerloaded status
   global devlist debugmenustrings runfilelist deviceselected
   global currentstart currentend

   if {$debug} {puts stdout "make-frame: frame=[self frame-name]"}
   if {$debug} {puts stdout "...devlist=$devlist"}

  set vx1 12c
  set vx1a 15c
  set vx2 19c
  set vx3 340
  set vx4 380
  set twid1 15
  set twid2 45

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/sort" \
      :icon-name "[self icon-title]" :resize F

  :panel BUTTONS  :layout B :padx 10 :pady 10
     :layout H; :row-gap 20; :col-gap 5
      :button ADD :label "Add" :w 2
      :button INSERT :label "Insert" :w 2
      :button REPLACE :label "Replace" :w 5
      :button REMOVE :label "Remove" :w 5
      :button RESETFILE :label "ResetLine" :w 6
      :button SAVE :label "Save" :w 4
      :button REFRESH :label "Refresh" :w 8 :x $vx4 
      :button MYHELP :label "Help" :w 7
   :panel RUNFILES :layout B :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 20
      :message LIST-HEADER :label "Output Runfiles" :bold T :x 280
      :list RUNLIST :rows 10 :list-width 500 :x 0 :exclusive T \
             :strings $runfilelist :see 0
     :next-row T
      :text VOLUME :label "Volume:" :w $twid2 :valuex $vx1
     :next-row T
      :text FILE :label "File:" :w $twid2 :valuex $vx1
     :next-row T
      :text STARTBLOCK :label "Start block:" :value $currentstart :valuex $vx1
    :button READRUNFILES :label "Load from outruninfo[set outchannel] File" :x $vx4
     :next-row T
      :text ENDBLOCK :label "End block:" :value $currentend :valuex $vx1
  :panel OPTIONS  :layout L :padx 10 :pady 10
     :layout H; :row-gap 20; :col-gap 10
      :message OUTPUT-HANDLER :label "Output handler: $handlerloaded" :bold T
    :message OUTPUT-TYPE :w 15 :x 150 :label "[set outputlabel($outchannel)]" :bold T
    :text OUTPUT-HANDLER-STATUS :label "Status:" :value $status :bold T \
       :x $vx4
      [:path OUTPUT-HANDLER-STATUS].entry configure -bd 0
    :next-row T
      :choice-stack DEVICE :label "Device" \
         :valuex $vx1a :w 6 :strings $devlist :value $deviceselected

   :show T
}

proc paint-handler-status {} {
   global status basedir
   set status 0
   :modify OUTPUT-HANDLER-STATUS :value $status
   }

proc repaint {} {
   global basedir handlerselected handlerloaded outputlabel outchannel
   global devlist deviceselected   
    global tracelist tracelevel

    :modify READRUNFILES :label "Load from outruninfo[set outchannel] File"
   :modify OUTPUT-TYPE :label "[set outputlabel($outchannel)]"
   :modify OUTPUT-HANDLER :label "Output handler:    $handlerloaded"
   paint-handler-status
   :modify DEVICE  :value $deviceselected
   paint-handler-status

   paint-runfiles
   }

proc paint-runfiles {} {
   global runfilelist runfileselected
   global currentvolume currentfile currentstart currentend

   set i 0
   set list ""
   foreach x $runfilelist {
       lappend list [format-runfile "$i [second $x] [third $x] [fourth $x] [fifth $x]"]
       incr i
   }
   set runfilelist $list

   :modify RUNLIST :empty T :strings $runfilelist :select $runfileselected :see $runfileselected
    if {$runfilelist != {}} {
   set runfile [lindex $runfilelist $runfileselected]

   set runfileselected [first $runfile]
   set currentvolume [second $runfile]
   set currentfile [third $runfile]
   set currentstart [fourth $runfile]
   set currentend [fifth $runfile]

   :modify VOLUME :value $currentvolume
   :modify FILE :value $currentfile
   :modify STARTBLOCK :value $currentstart
   :modify ENDBLOCK :value $currentend
   }
}

proc click-device {e} {
   global basedir preferences
   global deviceselected devlist outchannel
   frame-busy
   set deviceselected $e
   set d [lindex $devlist $e]

   set m ""
    set z [catch {writereg outputdevice[set outchannel] $d} m]
   if {$z!=0} then {set-footer $m; set deviceselected -1} \
   else {
       set preferences(outputdevice[set outchannel]) $d
       update_preferences
       clear-footer
      }
   repaint
   frame-idle
   }

proc click-volume v {
   global currentvolume
   set currentvolume [string toupper $v]
   }

proc no-spaces f {
    set s $f
    string trimleft $s
    while {[set i [string first " " $s]] != -1} {
	set s [string range $s 0 [expr $i -1]][string range $s [expr $i +1] end]
    }

    return $s
}

proc click-file f {
   global currentfile
# may not want tolower: may want asis for disc file
#    set currentfile  [no-spaces [string tolower $f]]
    set currentfile  [no-spaces $f]
   }

proc click-startblock s {
   global currentstart
   set currentstart  [string toupper $s]
   }

proc click-endblock e {
   global currentend
   set currentend  [string toupper $e]
   }

proc click-add {} {
   global runfilelist runfileselected
   global currentvolume currentfile currentstart currentend
   frame-busy
   if { $currentvolume == ""} {frame-idle; return}
   if { $currentfile == ""} {frame-idle; return}
   if { $currentstart == ""} {set currentstart 0}
   if { $currentend == ""} {set currentend 0}
   set runfileselected [llength $runfilelist]
   set l "$runfileselected $currentvolume $currentfile $currentstart $currentend"
   lappend runfilelist [format-runfile $l]
   paint-runfiles
   frame-idle
   }

proc click-insert {} {
   global runfilelist runfileselected
   global currentvolume currentfile currentstart currentend
   frame-busy
   if { $currentvolume == ""} {frame-idle; return}
   if { $currentfile == ""} {frame-idle; return}
   if { $currentstart == ""} {set currentstart 0}
   if { $currentend == ""} {set currentend 0}
   set l "$runfileselected $currentvolume $currentfile $currentstart $currentend"
    set runfilelist [linsert $runfilelist [expr $runfileselected +1] [format-runfile $l]]
   paint-runfiles
   frame-idle
   }

proc click-replace {} {
   global runfilelist runfileselected outhandler
   global currentvolume currentfile currentstart currentend
   frame-busy
   if {![llength $runfilelist]} then {
      click-add
      frame-idle
      return
      }
    if {$outhandler == 1} {set currentvolume DISC}
   set l "$runfileselected $currentvolume $currentfile $currentstart $currentend"
   set l [format-runfile $l]
   set runfilelist [lreplace $runfilelist $runfileselected $runfileselected $l]
   paint-runfiles
   frame-idle
   }

proc init-values {} {
   global debug deviceselected devlist outchannel

    set dev [readreg outputdevice[set outchannel]]
    set deviceselected [lsearch $devlist $dev]
    if {$deviceselected < 0} {set deviceselected 1}
   return 0
   }


proc start-handler args {
   global started
    make-frame
    init-values
    update idletasks
    click-refresh
    set started 1
   }
