# metronome2.tcl---GREAT Metronome module control frame
# 9/1/2004 removed the -16 when calculating average 
#          STARY now inverted. PJCS.


set frame "Metronome2"
set frametitle "Metronome2 Control Module"
set icontitle "Metronome2"

if {[info exists env(VMEServer)]} then {set resource $env(VMEServer)} else {set resource vme}
set VMEserver $resource
if {[info exists env(RDOUTserver)]} then {set resource $env(RDOUTserver)} else {set resource vme}
set RDOUTserver $resource

set modulename metronome2
set module 1

foreach i { 0 1 2 3 4 5 6 7 } { set InUse($i) 0}
foreach i { 0 1 2 3 4 5 6 7 } { set ModType($i) nodevice }
foreach i { 0 1 2 3 4 5 6 7 } { set ModNumber($i) 0 }
foreach i { 0 1 2 3 4 5 6 7 } { set AvgDelay($i) 0 }



# number of times to measure the delay in a channel
set AvgDelayLoop 100

# number of SYNC times ( 644uS ) it takes to write the new
# counter value to all possible targets, currently ~2 seconds. 
set MaxTime 3000

set VXIServer VXI

loadsource common.vme

set tdrlist {}
set patternlist {}

proc get-targets {} {
	global  tdrlist patternlist

	set tdrlist {}
	set ServerList {}
	set tdrtemplist [inform tdr32v inquire chanlist]
	set patternlist [inform pattern_register inquire modulelist]
	foreach vl $tdrtemplist {
		set tempserv [inform vxicontrol VXIModule2crate TDR32V$vl]
		set flag 0
		foreach trial $ServerList {
			if { [string compare $tempserv $trial ] == 1} { set flag 1 }
		}
		if { $flag == 0 } { 
			lappend tdrlist $vl 
			lappend ServerList $tempserv
		}
	}
#should now have two lists, a pattern list ( all the pattern registers ) and a list of one tdr from
# each VXI server 
}

proc channel_allocate {} {
# hunt through the list of targets, and allocate a device to each channel
	global tdrlist patternlist ModType InUse ModNumber VXIServer RegisterServer
	foreach i { 0 1 2 3 4 5 6 7 } { 
		log "Channel $i"
		foreach tdr $tdrlist {
			set RegisterServer $VXIServer
			set register TDR32V$tdr.CCR.bit0 
			write-register $register 1
			set result [ping_channel $i ]
			if { $result == 1 } then { 
				set ModType($i) tdr
				set ModNumber($i) $tdr
				set InUse($i) 1
				log "TDR32V #$tdr "
			}
			write-register $register 0
		}
		foreach pat $patternlist {
			set register pattern_register#$pat.InpCont
			write-vme-register $register 8
			set result [ping_channel $i ]
			if { $result == 1 } then { 
				set ModType($i) pattern
				set ModNumber($i) $pat
				set InUse($i) 1
				log "Pattern Register $pat"
			}
			write-vme-register $register 0
		}
	if { $InUse($i) == 0 } then { log "Channel not connected " }
	}

}

proc set-register {register v} {
    global modulename module
    write-vme-register $modulename#$module.$register $v
}

proc get-register {register} {
    global modulename module
    set v [read-vme-register $modulename#$module.$register]
    return $v
}

proc ltime {} {clock format [clock seconds] -format "%b %d %H:%M:%S"}

proc log {s} {:modify LOG :strings "[ltime]: $s\n"; update idletasks}

proc click-start  {v} {
#    if {![GetConfirmation "Start the Clock of"]} {return}
    set-register Start 1
}

proc click-stop   {v} {
#    if {![GetConfirmation "Stop the Clock of"]} {return}
    set-register Start 0
}

proc click-initialise {v} {

    if {![GetConfirmation Initialise]} {return}

    set-footer "Initialising the Metronome Module"

    log "Setting Initialise register to 0xaa"
    set-register Initialise 0xaa

    log "Setting Initialise register to 0x55"
    set-register Initialise 0x55

    log "Setting Initialise register to 0"
    set-register Initialise 0x0

    set-register Start 1     ;#  and start the clock

    set status [get-register Status]

    if {[expr $status & 1 ] == 0x1} {
       set-footer "OK: module initialised"
    } else {
       log "Initialise status is $status"
       set-footer "ERROR: initialisation failed"
    }
}

proc click-delay {ch vl} {set-register load.Ch[string range $ch 5 5] $vl}

proc click-inuse {ch vl} {global InUse; set InUse([string range $ch 5 5]) $vl}

proc click-counter {vl}  {set-register Counter $vl}

proc click-software {} {
   global modulename
   inform [set modulename]sftwre start
}

proc click-redisplay {} {
 	global InUse
    foreach i {0 1 2 3 4 5 6 7} {
        :modify DELAY$i :value [expr [get-register load.Ch$i] * 2.5]
	:modify INUSE$i :value $InUse($i) 
    }

    set chstatus [ get-register IStatus ]
    foreach i {0 1 2 3 4 5 6 7} {
        :modify CHERR$i :value [ expr ($chstatus >> $i) & 1]
    } 

    set vl [get-register Counter]
    :modify COUNTER :value $vl
    :modify STARTED :value [expr $vl & 1]

    set vl [get-register Status]
    :modify CHERR :value [expr ($vl >> 2) & 1]
    :modify CLKERR :value [expr ($vl >> 1) & 1]
    :modify INIT :value [expr $vl & 1]
}

proc click-clrerror {} {set-register Clear 0; click-redisplay}

proc click-calibrate {args} {
    global EGreport RegisterServer VXIServer InUse chanData tdrlist patternlist AvgDelay ModType

    log "\nStarting calibration"
    set-busy
    set-footer "Calibrating the system"

     set EGreport_SAVE $EGreport
     set EGreport -1

	get-targets
# set loop back mode on all channels according to type I.e. Pattern or VXI !
	channel_allocate
    set RegisterServer $VXIServer
	foreach tdr $tdrlist {
		set register TDR32V$tdr.CCR.bit0
		write-register $register 1
	}
	foreach pat $patternlist {
		set register pattern_register#$pat.InpCont
		write-vme-register $register 8
	}
#    write-registers TDR32V*.CCR.bit0 1
    set longest 0
    set longestChannel 0
    for {set Ch 0} {$Ch < 8}  {incr Ch} {
        if {$InUse($Ch) == 1} {
            log "Channel $Ch"
	    average-delay $Ch
            set nsec [expr $AvgDelay($Ch) * 2.5]
            log "delay for this channel is $AvgDelay($Ch) ($nsec nsecs)"
	    if { $AvgDelay($Ch) > $longest } { 
	    	set longestChannel $Ch 
	    	set longest $AvgDelay($Ch) 
	    }
        }
    }

# now work out the relative delays to make the values for the channels
# then divide by 2 to allow for the there and back part !
   for {set Ch 0} {$Ch < 8}  {incr Ch} {
        if {$InUse($Ch) == 1} {
		set delay [ expr (  $AvgDelay($longestChannel) - $AvgDelay($Ch)  ) / 2 ]
		if { [string compare $ModType($Ch) tdr ] == 0 } then {
			set delay [expr $delay + 4] 
		}
		set-register load.Ch$Ch $delay
        }
    }
    set RegisterServer $VXIServer
#   write-registers TDR32V*.CCR.bit0 0
# Clear loop back mode on all channels
	foreach tdr $tdrlist {
		set register TDR32V$tdr.CCR.bit0
		write-register $register 0
	}
	foreach pat $patternlist {
		set register pattern_register#$pat.InpCont
		write-vme-register $register 0
	}
	set-register ArmCalTest 0

    click-redisplay

    clear-busy
    set-footer "Calibration finished"

    set EGreport $EGreport_SAVE

    log "Calibration finished"
    insert-log "[date] TDR system calibrated"

}

proc ping_channel { ch } {

        set-register ChanSel $ch
	set-register ArmCalTest 1
        set delay [get-register Delay]
 	set-register ArmCalTest 0
        if {[expr $delay & 0xc0] == 128} then { return 1 } else {return 0  }
}

proc average-delay { ch } {
	global AvgDelay AvgDelayLoop

        set-register ChanSel $ch
	set average 0
 	for { set i 0 } { $i < $AvgDelayLoop } { incr i } {       
	    set-register ArmCalTest 1
            set delay [get-register Delay]
            if {[expr $delay & 0xc0] == 128} then {
                set vl [expr ($delay & 0x3f) ]
            } else {
               log "delay is invalid or out of range, check cable ;-)"
		set AvgDelay($ch) 0
		return
            }
 	    set average [ expr $average + $vl ]
	}
	set AvgDelay($ch) [ expr $average / $AvgDelayLoop ]

}

proc write-rdout-register {r v} {
	global RDOUTserver RegisterServer
	set RegisterServer_SAVE $RegisterServer
        set RegisterServer $RDOUTserver
	write-register $r $v
        set RegisterServer $RegisterServer_SAVE
}
	
proc synchronize-all { } {
	global VXIServer RegisterServer MaxTime
# get a list of all the modules using timestamps
	set tdrtemplist [inform tdr32v inquire chanlist]
	set patterntemplist [inform pattern_register inquire modulelist]
# fetch the current metronome counter value
	set MasterNow  [ get-register Counter  ]
	set MasterThen [ expr $MasterNow + $MaxTime ]
        set RegisterServer $VXIServer
	foreach i $tdrtemplist { 
		write-register TDR32V$i.TimeStampReLoad $MasterThen
	}
	foreach i $patterntemplist { 
		write-vme-register pattern_register#$i.LoadTime $MasterThen
	}

# We were (probably) called from sys but go back to sys for the Lyrtech call,
# this will keep context of calls to the LyrTech system in one place
        inform sys LyrTech_SetSync $MasterThen

# perhaps should check if it took too long here !
	set-register Counter $MasterThen
# the metronome will now issue the re-synch command at the right time.
# The value is also sent to the merge program and the click-da (GO) proc
# (see Custom_PostGo in sys.tcl) will tell merge to only allow data that
# is timestamped from that value onwards
# i.e. this only affects merge after the next GO
	write-rdout-register NetVar.DACQ.InitialTimeStamp $MasterThen
	set msgBody "Re-synchronising at [format 0x%x $MasterThen]"
	log "$msgBody"
insert-log "$msgBody"
        set msgClass 9; # Data Acquisition
        set msgLevel 3; # Information
        set msgSource [exec uname -n]
	catch {exec msgdefine $msgClass $msgLevel $msgSource "$msgBody"}
	foreach i $tdrtemplist { 
		log "TDR32V $i"
	}
	foreach i $patterntemplist { 
		log "Pattern Register $i "
	}
		log "Waiting for time to expire "
	while {  [get-register Counter]  < $MasterThen } {
		for { set i 0 } { $i < 1000 } { incr i } {} 
	}
	log "Re-synchronisation complete " 

	foreach i $tdrtemplist { 
		write-register TDR32V$i.CCR.bit2 1   ;# reset the SYNC error flag
		write-register TDR32V$i.CCR.bit2 0
	}
	foreach i $patterntemplist {
                inform pattern_register click-channel [expr $i-1]
		inform pattern_register click-input ICONT2 1     ;# reset the SYNC error flag
		inform pattern_register click-input ICONT2 0
	}
        click-clrerror
}

proc GetConfirmation {s} {
    set r [midas-confirmation "\
Be Careful!!.\n\n\
You have asked to $s the Metronome module.\n\
This can serious effect your experiment so please be sure you know what you are doing.\n\
Are you sure you wish to continue?" "No - Abort the request" "Yes I know what I am doing - Please continue"]
    return $r
}

proc start args {
   global started CurrentVMEServer modulename module

   if {$started == 0} then {
       make-frame
       EGbindings
   }
   set started 1
   open-frame

   clear-footer
   clear-busy
   set EGreport 0

   set CurrentVMEServer [VMEmodule2server $modulename#$module]
   click-redisplay
}

proc click-quit {} {close-frame}

proc finish args {destroy-frame}

proc click arg {
  
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]
   if {$c==":quit"} then {return [click-quit $f]}
   switch -glob $w {
       
      ENGINEERING       {click-$v}

      default           {
           set p "click-[lowercase $w]"
           if {[info procs $p] == "$p"} then {
              set argnum [llength [info args $p]]
              if {$argnum == 0} then {$p}
              if {$argnum == 1} then {$p "$v"}
              if {$argnum == 2} then {$p "$v" "$i"}
              if {$argnum > 2} then {insert-debug "click: $p expects too many args ($argnum)"}
           } else {
              insert-debug "click with argument \{$arg\}"
           }
       }
   }
}

proc make-frame {} {
  global env InUse
  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/vme_module" \
      :icon-name "[self icon-title]" 

   :panel MODULE  :layout T :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 10

     :col-gap 0
     :message M1 :label "Channels in Use" :valuex 16c
     foreach i {0 1 2 3 4 5 6 7} {
        :checkbox INUSE$i :label $i :value $InUse($i) :deaf T
     }

     :next-row T
     :message M4 :label "Channel Error  " :valuex 16c
     foreach i {0 1 2 3 4 5 6 7} {
        :checkbox CHERR$i :label $i  :deaf T
     }

     :next-row T
     :col-gap 10

     :message M2 :label "Delays (nsec)" :valuex 14c
     foreach i {0 1 2 3} { 
        :text DELAY$i  :label $i :w 6 :command click-delay
     }

     :next-row T

     :message M3 :label "" :valuex 14c
     foreach i {4 5 6 7} { 
        :text DELAY$i  :label $i :w 6 :command click-delay
     }

     :next-row T

     :text COUNTER :w 10 
     :checkbox STARTED :label "Counter Started" :deaf T

     :next-row T

     :checkbox CHERR   :label "Channel Error" :deaf T
     :checkbox CLKERR  :label "Clock PLL Error" :deaf T
     :checkbox INIT    :label "Initialised" :deaf T

     :next-row T

     :button INITIALISE  :label "Initialise"
     :button START       :label "Start Clock"
     :button STOP        :label "Stop Clock"
     :button CALIBRATE   :label "Calibrate"

     :next-row T
     :button CLRERROR   :label "Clear Error"
     :button SYNCHRONIZE :label "Synchronise All" :command synchronize-all

   :panel CLICK :layout B :padx 10 :after FOOTER
     :layout H; :col-gap 30; :row-gap 10
      :vskip 10
      :button REDISPLAY  :command click-redisplay
      :menubutton ENGINEERING :label "Engineering"  :menu [list software]

   :panel P3 :layout T :padx 10
     :galley LOG :rows 10 :cols 50 :font variable

     open-frame

     :galley_size LOG 10 50
}

proc click-configure {W} {:galley_resize LOG}



