# sharcdiag.tcl---SHARC Diagnostics frame

set debug 0
set started 0

set frame SHARCDIAG
set frametitle "Diagnostics for SHARCs, Collators & Other Merge Sources"
set icontitle "SHARCDiag"

#    current server resource
set resource "mem"
#    list of available server resources
set resources {}
#    number of available SHARC server resources
set numresources 0

# These values must match /eurogam/GREAT/include/great.h
set ITEM_BYTES 8
set CHUNK_LEN 8
set CHUNK_BYTES [expr $CHUNK_LEN * $ITEM_BYTES]
set COLLATE_BUFFER_LEN [expr 320 * 1024]
set COLLATE_BUFFER_CHUNKS [expr $COLLATE_BUFFER_LEN / $CHUNK_LEN]
set DMA_BUFFER_BYTES 0x00100000
set SHARC_BUFFER_BYTES [expr 128 * $ITEM_BYTES]

set Proc0 sharc0
set Proc1 sharc1
set Proc2 sharc2

# during debugging

set Proc0 sharc1
set Proc1 sharc2
set Proc2 sharc1


set ASPlistProc0 [list 0 1]
set ASPlistProc1 [list 2 3]
set ASPlistProc2 [list 4 5]

set CurrentASP 0
set CurrentSource 0

proc make-resource-list {} {
  global env resource resources numresources

   set z [catch {eg "(experiment ?resources)"} m]
   if {$z==0} then {set rs $m} else {set rs ""}

#    scan through resources and count instances of all mem register servers
   if {[info exists env(SHARCServer)] == 0} then {set role mem}  \
      else {set role $env(SHARCServer)}
   set count 0
   set rolecount(mem) 0
   foreach r $rs {
      set type [second [fourth $r]]
      set role [second [fifth $r]]
if {[second [first $r]] != "merge"} then {
      if {$type == "register" && $role == "mem"} then {incr count; incr rolecount(mem)}
}
   }

   if {[info exists env(SHARCServer)] == 0} then {set resource mem}  \
      else {set resource $env(SHARCServer)}
   set resources {}
   set numresources 0
   if {$rolecount(mem) > 0}   then {lappend resources mem; incr numresources}

   if {$count > 1 || $role != $resource} then {
#    next scan through resources and add instances of all mem register servers to the list
      foreach r $rs {
         set type [second [fourth $r]]
         set role [second [fifth $r]]
         set name [second [first $r]]
if {$name != "merge"} then {
         if {$type == "register" && $role == "mem" && $rolecount(mem) > 1}   then {lappend resources $name; incr numresources}
}
      }
   } else {
      if {[info exists env(SHARCServer)] == 0} then {set resources mem}  \
         else {set resources $env(SHARCServer)}
      set numresources 1
   }
   if {[info exists env(SHARCServer)] == 0} then {set resource [first $resources]}  \
      else {set resource $env(SHARCServer)}

   make-processor-list
}


proc click arg {
   global debug hsdir
   if {$debug} then {insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit $f; return}
   switch -glob $w {
      DET-*             {click-overlay $w $v}
      default    {puts stdout $arg}
   }
}

proc click-quit {f} {
   global env started
   if {$f=="SEL"} then {destroy-frame SEL; return}
   destroy-frame SEL
   close-frame
}

proc finish args {destroy-frame SEL; destroy-frame; set started 0}

set buffnames {"Collate Queue" "Collate Output" "Merge Queue" "Merge Output"}

set timing 0

set sname SHARC
set cname Collate
set mname Merge
set MergeProc merge

set ProcIndex 0
set CurrentProc sharc1
set ProcessorList {mem}
set HostList {mem}

set sharcmem 0

set StreamIndex 0
set LinkIndex 0
set GroupSize 6
set SrcGroup 0
set SrcStrings [list "0" "1" "2" "3" "4" "5"]
set meminc 0

set dumpstart 0
set dumpwords 64
set dumpdata  ""

set WordOrder 0

set memOffset 0x0000

#set PMC_SCTL		0xc2000000
#set SHARC_IOP		0xc2200000
set SHARC_PIO		0x00

set SHARC_IOP_OFFSET	0x00200000

set SHARC_SHARC_BASE	0x28400
#set LINK_OVERRUN_TOP	0x12
set PCTL_MBOX_BASE	0x40

# base address of link buffer 0
set SHARC_LK_BUFF_BASE	0x20400
set SHARC_LK_BUFF_BASE	0x27c00

set IOP_SYSCON		0x00000000
set ASP_SYSCON		0x0000e000
# for 48-bit access
set ASP_SYSCON_LK_RD	0x0000e028
	
for {set i 0} {$i<8} {incr i} {
   set PCTL_MBOX$i [format "0x%x" [expr $PCTL_MBOX_BASE + (4*$i)]]
}

set PCTL_PCI2SHARC 0x60
set PCTL_SHARC2PCI 0x64

set PPC_SHARC_BASE [format "0x%08x" [expr $SHARC_IOP_OFFSET + ($SHARC_SHARC_BASE << 2)]]
set PPC_LK_BUFF_BASE [format "0x%08x" [expr $SHARC_IOP_OFFSET + ($SHARC_LK_BUFF_BASE << 2)]]

for {set i 0} {$i<4} {incr i} {
#   set LINK_OVERRUN$i [format "0x%02x" [expr ($LINK_OVERRUN_TOP + $i) << 2]]
   set LINK_BUFFER$i [format "0x%02x" [expr ($i * 256) << 2]]
}

#set DMA_BUF [format "0x%08x" [expr 64*1024*1024 - 64*1024]]
set DMA_BUF 0

set barRegisters [list \
   "_SCTL generic -n -t int -fx" \
   "_PCTL generic -n -t int -fx" \
   "_DMA generic -n -t int -fx" \
]

set DMARegister   "DMABuffer bMEM dma -a 0 -o$memOffset -r0" 

#   "Link.OverRun0 xMEM sctl -a $PPC_SHARC_BASE -o$LINK_OVERRUN0 -zd -r1" \
#   "Link.OverRun1 xMEM sctl -a $PPC_SHARC_BASE -o$LINK_OVERRUN1 -zd -r1" \
#   "Link.OverRun2 xMEM sctl -a $PPC_SHARC_BASE -o$LINK_OVERRUN2 -zd -r1" \
#   "Link.OverRun3 xMEM sctl -a $PPC_SHARC_BASE -o$LINK_OVERRUN3 -zd -r1" \

set ASPC2Registers [list \
   "DMABuffer bMEM dma -a $DMA_BUF -o$memOffset -r0"  \
   "LinkBuffer0 bMEM sctl -a $PPC_LK_BUFF_BASE -o$LINK_BUFFER0 -zd -r1"  \
   "MailBox0 xMEM pctl -a $SHARC_PIO -o$PCTL_MBOX0 -r1 -ro" \
   "MailBox1 xMEM pctl -a $SHARC_PIO -o$PCTL_MBOX1 -r1 -ro" \
   "MailBox2 xMEM pctl -a $SHARC_PIO -o$PCTL_MBOX2 -r1 -ro" \
   "MailBox3 xMEM pctl -a $SHARC_PIO -o$PCTL_MBOX3 -r1 -ro" \
   "MailBox4 xMEM pctl -a $SHARC_PIO -o$PCTL_MBOX4 -r1 -ro" \
   "MailBox5 xMEM pctl -a $SHARC_PIO -o$PCTL_MBOX5 -r1 -ro" \
   "MailBox6 xMEM pctl -a $SHARC_PIO -o$PCTL_MBOX6 -r1 -ro" \
   "MailBox7 xMEM pctl -a $SHARC_PIO -o$PCTL_MBOX7 -r1 -ro" \
   "PCI2Sharc xMEM pctl -a $SHARC_PIO -o$PCTL_PCI2SHARC -r1" \
   "Sharc2PCI xMEM pctl -a $SHARC_PIO -o$PCTL_SHARC2PCI -r1" \
   "Sharc.IOP_SysCon xMEM sctl -a $SHARC_IOP_OFFSET -o$IOP_SYSCON -m16 -zx -r1" \
   "Sharc.Memory xMEM sctl -a $PPC_SHARC_BASE -o0 -zx -r1" \
   "Link.Buffer0 xMEM sctl -a $PPC_LK_BUFF_BASE -o$LINK_BUFFER0 -zd -r1" \
   "Link.Buffer1 xMEM sctl -a $PPC_LK_BUFF_BASE -o$LINK_BUFFER1 -zd -r1" \
   "Link.Buffer2 xMEM sctl -a $PPC_LK_BUFF_BASE -o$LINK_BUFFER2 -zd -r1" \
   "Link.Buffer3 xMEM sctl -a $PPC_LK_BUFF_BASE -o$LINK_BUFFER3 -zd -r1"
]

set table(DET-P2S)      {write-sharc PCI2Sharc}
set table(DET-S2P)      {write-sharc Sharc2PCI}

proc click-on {w v} {
	global debug table  

	if {$debug} then {puts stderr "click on $w $v"}
	set func [first $table($w)]
	set reg  [second $table($w)]

	$func $reg $v
}

proc click-overlay {w v} {
   global memOffset meminc StreamIndex

   switch -glob $w {
      DET-SOURCE	{click-source $v}
      DET-MEMADDR	{click-memaddr $v;update-sharcmem}
      DET-STREAM	{click-stream $v}
      DET-LINK		{click-link $v}
      DET-SRCGRP	{click-srcgroup $v}
      DET-MEMOFFSET	{click-offset $v}
      DET-INCR		{click-incr $v}
      DET-EMPTY		{:modify DET-MEMDISPLAY :empty T;global dumpdata; set dumpdata  ""}
      DET-REDISPLAY	{click-redisplay}
      DET-P2S		-
      DET-S2P		{click-on $w $v}
      default	{puts stdout "$w $v"}
   }
}

proc click-redisplay {} {
   global  frame
   if {![winfo ismapped [:path $frame]]} then {return}
   make-processor-list
   update-sharc
#   update-sharcmem
   update-others
   update-datacounters
}

proc update-sharcmem {} {
   global sname sharcmem PPC_SHARC_BASE
   erswta $sname.Sharc.Memory -o [expr $PPC_SHARC_BASE + ($sharcmem << 2)]
   set v [ersread $sname.Sharc.Memory]
   :modify DET-MEMCONT :value $v
}

proc click-source {n} {
  global StreamIndex LinkIndex SrcGroup ProcIndex CurrentProc CurrentASP CurrentSource GroupSize

  if {$n < 0} {set n 0}
  if {$n >= $GroupSize} {set n [expr $n % $GroupSize]}
  set src [expr $SrcGroup * $GroupSize + $n]

  if {$SrcGroup == 0} then {
    set ProcIndex -1
    for {set i 0} {$i < 3} {incr i} {
      global ASPlistProc$i
      set a [lsearch [set ASPlistProc$i] $src]
      if {$a != -1} {set ProcIndex $i; break}
    }
    if {$ProcIndex == -1} {
      set ProcIndex 0
      set src 0
      :modify DET-SOURCE :value $src
    }

    make-processor-list
    if {$ProcIndex != $i} {
      set ProcIndex 0
      set src 0
      :modify DET-SOURCE :value $src
    }

    set CurrentASP $src
    do-define-asp-registers $CurrentASP
  }
  set CurrentSource $src
  click-redisplay
}

proc click-memaddr {v} {
   global sharcmem
   set sharcmem $v
}

proc do-memmod {v} {
   global sharcmem
   incr sharcmem $v
   :modify DET-MEMADDR :value [format 0x%08x $sharcmem]
}

proc click-stream {n} {
   global StreamIndex LinkIndex SrcGroup dumpwords

   if {$n < 2 && $SrcGroup} then {
      set n 2
   }

   :modify DET-STREAM :value $n
   if {$n == $StreamIndex} then {
#     update-others
      return
   }
   set StreamIndex $n

   set label "Read Buffer"
   set dumpwords 64
   zero_offset

   :modify DET-RDBUFF :label $label

   if {$StreamIndex} {
     :modify DET-LINK :value -1
     :modify DET-LINK :inactive T
   } else {
     click-srcgroup 0
   }

   update-others
}

proc click-link {n} {
  global LinkIndex

  set LinkIndex $n
}

proc click-srcgroup {n} {
  global StreamIndex LinkIndex SrcGroup SrcStrings GroupSize CurrentSource

  if {$n < 0 || $n > 9} {
    :modify DET-SRCGRP :value $SrcGroup
     return
  }
  set SrcGroup $n

  set SrcStrings {}
  for {set i 0} {$i < $GroupSize} {incr i} {
     set item [expr ($SrcGroup * $GroupSize) + $i]
     set str [format "%2d" $item]
     lappend SrcStrings $str
  }
  :modify DET-SOURCE :strings $SrcStrings
  :modify DET-SRCGRP :value $SrcGroup

  if {$SrcGroup} {
    :modify DET-LINK :value -1
    :modify DET-LINK :inactive T
  } else {
    :modify DET-LINK :inactive F
    :modify DET-LINK :value  $LinkIndex
#    click-link $LinkIndex
  }
  click-source $CurrentSource
  click-stream $StreamIndex
}

proc click-offset {v} {
   set_offset $v
}

proc click-incr {v} {
   global StreamIndex meminc
   set m 0
      set meminc $v
      set m $v
   :modify DET-INCR :value $m
}

proc set_offset {v} {
   global memOffset StreamIndex
   set m 0x0000
      set memOffset [format 0x%04x $v]
      set m $memOffset
   :modify DET-MEMOFFSET :value $m
}

proc zero_offset {} {set_offset 0}

set skiptitle 1

proc format_the_dump {} {
  global dumpdata dumpstart dumpwords

#    format TDR data held in global variable dumpdata

   if {$dumpdata == ""} {
      return
   }

   set-footer "formatting data dump"

   set frame "dump"
   set win [self frame-name].$frame
   :frame $win :label  "Formatted Data Dump"  :transient T
   :layout T

   :galley G :rows 50 :cols 75 :strings "" :font fixed
   :show T
   :modify G :strings "dump starting at word offset $dumpstart for $dumpwords words\n\n"

   set waslink 0
   set wastrace 0
   set lasttype 0
   set lastcode 0
   set trsamples 0
   set trlength 0
   set skiptitle 1

   set adctype "        "
   set adcnum "      "

   for {set i 0} {$i < $dumpwords} {incr i} {
      if {$lasttype == "0" && $wastrace == "1"} {
        if {!$skiptitle} {:modify G :strings "[format "%s %d %s" "Skipping" $trsamples "samples"] \n"; set skiptitle 1}
        incr trlength -1
        if {$trlength > 0} {continue} {set wastrace 0}
      }

      set word  [lindex $dumpdata $i]
      set type  [expr $word >> 30 & 3]

      switch $type {
         0 {
            set not_sample 1
            if {$lasttype == "0"} then {
               set mess "ALIGNMENT ERROR?"
               set not_sample [expr $word & 0xc000c000]
               if {$not_sample == 0} {set mess "ERROR (OR SAMPLE DATA)"}
               :modify G :strings " $mess \n"
            }

            set title2 ""
            if {[expr $word  & 0xf0000000] == 0} then {
               if {$waslink != "1"} {
                 set title "Timestamp"
               } else {
                 set title "Sequence Number"
                 set title2 [expr $word]
               }
               if {$lasttype == "2"} {
                 if {$lastcode == "7" || $lastcode == "4" || $lastcode == "3" || $lastcode == "2"} {
                  set title2 [format "%05x%07x" $info $word]
                 }
               }
            } else {
               if {$not_sample == 0} {set title "Possible Sample Data"} {set title "Unrecognised"}
            }
            :modify G :strings "[format "%08x %28s %13s %6s  %s" $word $title $title2 $adcnum $adctype] \n"
            set adctype "        "
            set adcnum "      "
            set waslink 0
           }

         1 {
            if {$lasttype != "0"} then {
               :modify G :strings "ALIGNMENT ERROR \n"
            }
            set id    [expr $word >> 16 & 0xfff]
            set trsamples [expr $word & 0xffff]
            set trlength [expr $trsamples / 2]
            set title "Trace Buffer"
            set title2 [format "Module %3d" [expr $word >> 21 & 0x7f]]
            if {[expr $word >> 20 & 1]} {set dts "Raw Data"} {set dts "Trace Data"}
            set title3 [format "ADC %2d" [expr $word >> 16 & 0xf]]
            set waslink 0
            set wastrace 1

            :modify G :strings "[format "%08x %d %d %d %03x %04x %14s %11s %7s %11s" $word $type 0 0 $id $trsamples $title $title2 $title3 $dts] \n"
            set skiptitle 0
           }

         2 {
            if {$lasttype != "0"} then {
               :modify G :strings "ALIGNMENT ERROR \n"
            }

            set mod   [expr $word >> 24 & 0x3f]
            set code  [expr $word >> 20 & 0xf]
            set info  [expr $word & 0xfffff]
            set title3 ""
            switch $code {
            1   {
                 set title "Pileup"
                 set title2 [format "Module %3d" $mod]
                 set title3 [format "Channel %2d" $info]
                 set waslink 0
                }

            2   {
                 set title "Pause"
                 set title2 [format "Module %3d" $mod]
                 set waslink 0
                }

            3   {
                 set title "Resume"
                 set title2 [format "Module %3d" $mod]
                 set waslink 0
                }

            4   {
                 set title "SYNC100"
                 set title2 [format "Module %3d" $mod]
                 set waslink 0
                }

            7   {
                 set title "HeartBeat"
                 set title2 [format "Module %3d" $mod]
                 set waslink 0
                }

           13   {
                 set title "Trigger Seq"
                 set title2 [format "Module %3d" $mod]
                 set waslink 0
                }

            14  {
                 set title "Data Link"
                 set title2 [format "Number %d" $info]
                 set waslink 1
                }

            15  {
                 set title "Sharc Link"
                 if {$mod != "63"} {
                   set title2 "Faulty"
                   set waslink 0
                 } else {
                   set title2 [format "Number %d" $info]
                   set waslink 1
                 }
                }

            default  {
                 set title "Unknown Info"
                 set title2 ""
                 set waslink 0
                }
              }
            set lastcode $code
            :modify G :strings "[format "%08x %d %2d %2d  %05x %14s %11s %11s" $word $type $mod $code $info $title $title2 $title3] \n"
            set adctype "        "
            set adcnum "      "
           }

         3 {
            if {$lasttype != "0"} then {
               :modify G :strings "ALIGNMENT ERROR \n"
            }

            set title "ADC Data"
            set fail  [expr $word >> 29 & 1]
            if {$fail} {set f "Fail"} {set f "    "}
            set veto  [expr $word >> 28 & 1]
            if {$veto} {set v "Veto"} {set v "    "}
            set id    [expr $word >> 16 & 0xfff]
            set data  [expr $word & 0xffff]

            set title2 [format "Module %3d" [expr $word >> 21 & 0x7f]]
            set title3 [format "Channel %2d" [expr $word >> 16 & 0x1f]]

            if {!$fail && !$veto} {
               if {[expr $word >> 20 & 1]} {set adctype "Baseline"} {set adctype "Energy"}
               set adcnum [format "ADC %2d" [expr $word >> 16 & 0xf]]
            } {
               set adctype "        "
               set adcnum "      "
            }

            :modify G :strings "[format "%08x %d %d %d %03x %04x %14s %11s %11s %5s %s" $word $type $fail $veto $id $data $title $title2 $title3 $f $v] \n"
            set waslink 0
           }

          default  {
            set title "Unrecognised"
            set title2 ""
            :modify G :strings "[format "%08x %29s" $word $title] \n"
            set waslink 0
            set adctype "        "
            set adcnum "      "
           }
      }
      set lasttype $type
   }
   if {$lasttype == "0" && $wastrace == "1"} {
        if {!$skiptitle} {:modify G :strings "[format "%s %d %s" "Skipping" $trsamples "samples"] \n"; set skiptitle 1}
   }
   clear-footer
}

proc displayDump {header addr inc} {
   global dumpdata dumpwords

   :modify DET-MEMDISPLAY :strings "\n$header"

   for {set i 0} {$i<$dumpwords} {incr i} {
      if {![expr $i % 8]} {
         :modify DET-MEMDISPLAY :strings "\n"
         :modify DET-MEMDISPLAY :strings [format "0x%06x: " $addr]; incr addr $inc
      }
      if {[set data [lindex $dumpdata $i]] == ""} {break;}
      :modify DET-MEMDISPLAY :strings [format "%08x " $data]
   }
   :modify DET-MEMDISPLAY :strings "\n"
}


set MaxTries 19
proc wait_for_data {args} {
    global CurrentProc MaxTries EGreport

    set tries 1
    set TASK EXEC

    while {$tries < $MaxTries} {

        if {[llength $args] == 0} then {
            if {$EGreport != -1} then {set-footer "$CurrentProc: Waiting for action to complete ($tries)"}
        } else {
            if {$EGreport != -1} then {set-footer "$CurrentProc: [first $args] ($tries)"}
        }

        set z [catch {ersread [first $args].EXEC.Action} m ]

        if {$z != 0} then {
            midas-warning "$CurrentProc: Unable to confirm state of action\n$m"
            clear-footer
            return 1
        }
        if {$m == 0} then {
            if {$EGreport != -1} clear-footer
            return 0
        }
        incr tries
        after 500
    }

    midas-warning "$CurrentProc: Action has not completed\nGiving up waiting"
    clear-footer
    return 1
}

proc get_data {args} {
    global StreamIndex CurrentProc MergeProc LinkIndex CurrentASP SrcGroup WordOrder

#    first	action
#    second	parameter
#    third	signal number

      set TempProc $CurrentProc
# 0 Collate Link Queue
# 1 Collate Output
# 2 Merge Link/ASP Queue
# 3 Merge Output
      switch $StreamIndex {
         0 {set r Collate.Asp$CurrentASP; set s $r}
         1 {set r Collate.Asp$CurrentASP; set s $r}
         2 {set r NetVar; set s SigTask; set CurrentProc $MergeProc}
         3 {set r NetVar; set s SigTask; set CurrentProc $MergeProc}
         default {set r ""}
      }
      if {"$r" == ""} {set CurrentProc $TempProc; return}
      if {$SrcGroup != 0 && $CurrentProc != $MergeProc} then {
        midas-warning "Software Logic Error: Trying to read ASP data from the Merge\n"
       clear-footer
        return
      }

        if {[llength $args] > 2} {
            set sig [third $args]
        } else {
            set sig 1
        }

        set pid [ersread $r.EXEC.PID]
        if {[llength $args] > 1} {erswrite $r.EXEC.Parameter "[second $args]"}
        erswrite $r.EXEC.Action [first $args]
        erswta $s.EXEC.Usr$sig -t $pid
        erswrite $s.EXEC.Usr$sig 0

        if {$CurrentProc == $MergeProc} {set asp ""} {set asp "Asp$CurrentASP"}
        wait_for_data $r "Getting Raw Data from $CurrentProc $asp"
        append data [ersread $r.EXEC.Response]
        if {$StreamIndex == 3} {set WordOrder [ersread $r.DACQ.GREATWordOrder]}

      set CurrentProc $TempProc
    return $data
}

proc Dump_Raw_Data {args} {
   global dumpdata dumpwords meminc memOffset WordOrder
   global StreamIndex LinkIndex SrcGroup CurrentASP CurrentSource

set dumpwords 64

set dumpdata {}

# command parameter signal
# 19 -read raw data, 12 - from GREAT at word offset, SIGUSR2
# 0 Collate Link(SHARC) Queue
# 1 Collate Output
# 2 Merge Link/ASP Queue
# 3 Merge Output
      switch $StreamIndex {
         0 {set Index $LinkIndex; set io 0; set type 12}
         1 {set Index 0; set io 1; set type 12}
         2 {set Index $CurrentSource; set io 0; set type 13}
         3 {set Index 0; set io 1; set type 13}
         default {set Index 0; set io 0; set type 12}
      }
     if {[llength $args]} {set start -1} {set start [expr $memOffset]}
     set buffer [get_data 19 "$type $io $start $Index" 2]
     set data [lrange $buffer 2 end-2]
# for integrity checking
     set leng [expr 0x[lrange $buffer end-1 end-1]]
     set len [llength $data]

     foreach word $data {
         set xword 0x$word
         if {$xword == 0xaa5555aa} then {
             break
         }
         if {$xword == "0x"} then {
             break
         }
         append dumpdata "$xword "
     }
#insert-debug "offset [first $buffer]"
#   if {[llength $args]} then {
      set_offset 0x[first $buffer]
#   }
   set header "Memory Offset $memOffset" 
   if {$StreamIndex == 3 && $WordOrder == 1} then {
      set header [format "%s\n%s" "Output word order is timestamp first. Word pairs have been swapped for display" $header]
   }
   displayDump $header $memOffset 32

   if {$meminc != 0} then {
      set_offset [expr $memOffset + 256]
   }
}

proc click-readBuff {} {
   global StreamIndex LinkIndex

      global CurrentProc MergeProc CurrentASP
      set TempProc $CurrentProc
      Dump_Raw_Data
      set CurrentProc $TempProc

   update-sharc
   update-datacounters
}

proc click-readCurr {} {
   global StreamIndex LinkIndex

      global CurrentProc MergeProc CurrentASP
      set TempProc $CurrentProc
      Dump_Raw_Data 1
      set CurrentProc $TempProc

   update-sharc
   update-datacounters
}

proc click-define-asp-registers {} {
   global CurrentASP
   do-define-asp-registers $CurrentASP
}

proc write-sharc {r v} {
   global debug sname

   if {$debug} then {puts stderr "write-sharc $sname.$r $v"}
   erswrite $sname.$r $v
}

proc start {} {
   global started frame CurrentASP
   if {$started==0} then {make-frame} else {open-frame $frame}
   make-resource-list
   catch {do-define-asp-registers $CurrentASP}
   click-redisplay   
}

proc make-frame {} {
   global env frame started
   global ProcessorList buffnames StreamIndex SrcStrings SrcGroup memOffset

      set label "Read Buffer"

   :frame [self frame-name] :label "[self frame-title]" :show-footer T  :resize F \
        :icon-image "@$env(TCLHOME)/bitmaps/spectrum" \
        :icon-name "[self icon-title]"

# N.B.
# MailBox 0 PCI DMA buffer number
# MailBox 1 Number of buffers sent
# MailBox 2 Address of LUT
# MailBox 3 Date & Version
# MailBox 4 PCI DMA status
# MailBox 5 Unallocated
# MailBox 6 Unallocated
# MailBox 7 Unallocated
#      :text DET-MBOX0  :label "PCI DMA buffer number:" :deaf T :valuex 170 :w 10
#      :text DET-MBOX4  :label "PCI DMA status:" :deaf T :valuex 115 :w 10
#      :next-row T
#      :text DET-MBOX1  :label "Number of buffers sent:" :deaf T :valuex 170 :w 10
#      :text DET-MBOX5  :label "MailBox 5:" :deaf T :valuex 115 :w 10
#      :next-row T
#      :text DET-MBOX2  :label "Address of LUT:" :deaf T :valuex 170 :w 10
#      :text DET-MBOX6  :label "MailBox 6:" :deaf T :valuex 115 :w 10
#      :next-row T
#      :text DET-MBOX3  :label "Date & Version:" :deaf T :valuex 170 :w 10
#      :text DET-MBOX7  :label "MailBox 7:" :deaf T :valuex 115 :w 10


   :panel P20  :layout T :padx 10 :pady 10
      :layout H; :row-gap 10; :col-gap 10
      :text DET-PROCESSOR :label "Register Server" :w 8 :deaf T
      :hskip 32
      :text DET-HOST :label "Host Name" :w 8 :deaf T
      :hskip 32
      :choice DET-SOURCE :label "Source" :w 2 :strings $SrcStrings :exclusive T
      :number DET-SRCGRP :label "Source Group" :w 2 :value $SrcGroup
      :next-row T

   :panel MBOXES  :layout T :padx 10 :pady 10
      :layout H; :row-gap 0; :col-gap 10
      :text DET-MBOX0  :label "MailBox 0:" :deaf T :valuex 68 :w 10
      :text DET-MBOX1  :label "Buffers sent:" :deaf T :valuex 98 :w 10
      :text DET-MBOX2  :label "Buffers read:" :deaf T :valuex 98 :w 10
      :text DET-MBOX3  :label "Date/Version:" :deaf T :valuex 98 :w 10
      :next-row T
      :text DET-MBOX4  :label "MailBox 4:" :deaf T :valuex 68 :w 10
      :text DET-MBOX5  :label "DMA buffer (P):" :deaf T :valuex 98 :w 10
      :text DET-MBOX6  :label "DMA buffer (S):" :deaf T :valuex 98 :w 10
      :text DET-MBOX7  :label "MailBox 7:" :deaf T :valuex 98 :w 10

   :panel DOORBELLS  :layout T :padx 10 :pady 10
      :layout H; :row-gap 0; :col-gap 10
      :message DET-DOORBELLS :label "Doorbells:" :bold T
      :text DET-P2S  :label "PCI to Sharc:" :valuex 150 :w 10
      :col-gap 20
      :text DET-S2P  :label "Sharc to PCI:" :valuex 84 :w 10

   :panel DATACOUNTERS  :layout T :padx 10 :pady 10
      :layout H; :row-gap 0; :col-gap 10
      :message DET-COUNTERS :label "Items Received:" :bold T
      :text DET-ASP-RECEIVED  :label "ASP:" :deaf T :valuex 118 :w 10
      :text DET-MRG-RECEIVED  :label "Merge:" :deaf T :valuex 98 :w 10
      :text DET-BUFS-USED  :label "Link Buffers in use:" :deaf T :w 10

   :panel DISPLAY :layout B :padx 10 :after FOOTER
      :layout H; :row-gap 10; :col-gap 15
      :choice-stack DET-STREAM :label "Read From:" :strings $buffnames :value $StreamIndex
      :choice DET-LINK :label "SHARC Link" :strings [list 0 1 2 3 4 5] :exclusive T
      :text DET-MEMOFFSET :label "Offset:" :w 8 :value "$memOffset"
# :x 200
      :button DET-ZERO :label "Zero Offset" :command "zero_offset"
      :checkbox DET-INCR :label "Increment Offset" :value 0
#      :text DET-OVERRUN :label "Link Overruns:" :w 6 :deaf T
      :next-row T
      :galley DET-MEMDISPLAY :rows 11 :cols 90 :font fixed

   :panel CLICK :layout B :padx 10 :pady 10 :after FOOTER
      :layout H; :row-gap 15; :col-gap 15
      :button DET-REDISPLAY :label "Redisplay"
      :button DET-RDBUFF :label $label :command "click-readBuff"
      :button DET-RDCURR :label "Read Current" :command "click-readCurr"
      :button DET-FORMAT :label "Format Data"  :command format_the_dump
      :button DET-EMPTY  :label "Empty memory display"
      :button DET-DEFINE :label "Define Registers" :command "click-define-asp-registers"

   :panel P21 :layout T :padx 5 :pady 5

  :show T

 set started 1

}

proc get-buffers-sent {args} {
   global sname CurrentProc EGallow
   
   if {[llength $args]} {set p [first $args]} {set p mem}

   set keepproc $CurrentProc
   set CurrentProc $p

   set z [catch {ersread $sname.MailBox1} m]
   if {$z != 0 && $m != "0x10002"} then {
      midas-error $m
   }
   set CurrentProc $keepproc
   return $m
}

# This procedure writes 1 to the Sharc doorbell - use with care
proc initialise-links {args} {
   global sname CurrentASP CurrentProc EGreport EGallow

   catch {do-define-asp-registers $CurrentASP}
   if {[llength $args]} {set p [first $args]} {set p mem}

   set keepproc $CurrentProc
   set CurrentProc $p

   set egallow $EGallow; set EGallow 0x10002
   set egreport $EGreport; set EGreport -1
   set z [catch {write-sharc PCI2Sharc 1} m]
   if {$z != 0} then {
      midas-error $m
   }
   set EGreport $egreport
   set EGallow $egallow
   set CurrentProc $keepproc
   return
}

proc update-sharc {} {
   global sname EGallow

   for {set i 0} {$i<8} {incr i} {
      set v [ersread $sname.MailBox$i]
      :modify DET-MBOX$i  :value $v
   }
   set v [ersread $sname.PCI2Sharc]
   :modify DET-P2S  :value $v
   set v [ersread $sname.Sharc2PCI]
   :modify DET-S2P  :value $v
}

proc update-datacounters {} {
   global EGallow cname CurrentASP
   global CurrentProc MergeProc mname

   set v [ersread $cname.Asp$CurrentASP.Stats.TotalCount]
   :modify DET-ASP-RECEIVED  :value $v

   set TempProc $CurrentProc
   set CurrentProc $MergeProc
   set v [ersread NetVar.DACQ.Stats.TotalCount]
   :modify DET-MRG-RECEIVED  :value $v

   erswta NetVar.DACQ.Stats.BuffersInUse "-i [expr 20 + $CurrentASP]"
   set v [ersread NetVar.DACQ.Stats.BuffersInUse]
   :modify DET-BUFS-USED  :value $v
   set CurrentProc $TempProc
}

proc update-others {} {
   global memOffset meminc sname StreamIndex

      set o "******"
      set m $memOffset
      set i $meminc

   :modify DET-MEMOFFSET  :value $m
   :modify DET-INCR  :value $i
}

proc make-processor-list {} {
  global started
  global ProcessorList HostList resource
  global CurrentProc CurrentHost ProcIndex

#    find processors available

   if {[catch {eg "(experiment ?resources)"} m]} then {
      midas-warning "no experiment connected!!!"
      set ProcessorList {mem1}
      set HostList {mem1}
      set ProcIndex 0
   } else {
      set ProcessorList [make-sharc-resource-list "name" $m $resource]
      set HostList  [make-sharc-resource-list "host" $m $resource]
      if {$ProcIndex >= [llength $ProcessorList]} then {set ProcIndex 0}
   }
#    sort lists into sensible order
   set proclist [lsort $ProcessorList]
   set hostlist {}
   foreach c $proclist {
      lappend hostlist [lindex $HostList [lsearch $ProcessorList $c]]
   }
   set ProcessorList $proclist
   set HostList $hostlist
   set CurrentProc     [lindex $ProcessorList $ProcIndex]
   set CurrentHost     [lindex $HostList $ProcIndex]

   if {$started} then {
         :modify DET-PROCESSOR  :value $CurrentProc
#      :modify DET-PROCESSOR :strings $ProcessorList :value $ProcIndex
         :modify DET-HOST  :value $CurrentHost
   }
}

proc make-sharc-resource-list {class rlist resource} {
   set ll {}
   foreach item $rlist {
      set l [find-resource $class $item $resource]
      if {$l!={}} then {lappend ll $l}
   }
   return $ll
}

proc find-resource {class ps resource} {
   set l {}
   set role {}; set name {}; set host {}
   foreach p $ps {
      switch [first $p] {
         name {set name [second $p]}
         role {set role [second $p]}
         host {set host [second $p]}
         type {set type [second $p]}
         default {}
      }
   }
   if {$role==$resource && $type=="register" && $name!="merge"}  then  {set l [set $class]}
   return $l
}

proc ersdefine {register class} {
   global CurrentProc
   EG "register :name '/$CurrentProc/$register' :class '$class' !new"
}

proc ersinquire {register} {
   global CurrentProc
   eg "(register :pattern '/$CurrentProc/$register' ?names)"
}

proc erswta {register args} {
   global CurrentProc
   EG "register  :name '/$CurrentProc/$register' :object '$args'"
}

proc ersread {register} {
   global CurrentProc EGreport

   set egreport $EGreport; set EGreport -1
   set res [EG "register  :name '/$CurrentProc/$register' ?value"]
   set EGreport $egreport
   return [string trimright $res]
}

# Beware this uses :pattern - so it could write to ALL hosts with 1 call
proc erswrite {register val} {
   global debug
   global CurrentProc

   if {$debug} then {puts stderr "erswrite: EG register  :pattern '/$CurrentProc/$register' :value '$val'"}
   EG "register  :pattern '/$CurrentProc/$register' :value '$val'"
}

proc define-register {name class attributes} {
   global sname  EGreport
   global CurrentProc

   set register $sname.$name

   set z [catch {ersinquire $register} r]
   if {$z != 0} then {return}
   if {$r==""} then {
      if {$EGreport>=0} then {set-footer "defining register $register"}
      if {[catch {ersdefine $register $class} m]} then {
         midas-error $m
      }
   }
   set z [catch {ersinquire $register} r]
   if {$r == $register} then {erswta $register "$attributes"}
   clear-footer
}

proc define-asp-registers {p a} {
   global sname CurrentProc barRegisters ASPC2Registers
   set keepproc $CurrentProc
   set CurrentProc $p

   foreach r $barRegisters {
      set attr [list [third $r] asp[set a][first $r] [fourth $r] [fifth $r] [sixth $r]]
      define-register asp[set a][first $r] [second $r] $attr
   }
   set pctl [ersread $sname.asp[set a]_PCTL]
   set sctl [ersread $sname.asp[set a]_SCTL]
   set dma  [ersread $sname.asp[set a]_DMA]

   foreach r $ASPC2Registers {
      set attr [lreplace $r 0 2]
      set addr [expr $[third $r] + [second $attr]]
      set attr [lreplace $attr 1 1 $addr]
      define-register [first $r] [second $r] $attr
   }

   set CurrentProc $keepproc
}

proc do-define-asp-registers {a} {
   global ProcessorList CurrentProc sname ASPC2Registers EGreport
   set egreport $EGreport; set EGreport -1

   define-asp-registers $CurrentProc $a

   set EGreport $egreport
}
