#
# Implements slightly more efficient way of reading the Stats & Rates arrays
# Makes a single read call for the array not one per item
# N.B. The Stats & Rates tcl "arrays" are now lists not arrays
# Not even sure they're needed at all
#
# Implements server choice - merge or one of the asps
#

set RDOUTserverList [inform sys inquire RDOUTserverList]
set RDOUTserver [first $RDOUTserverList]

set MergeRegPrefix "NetVar.DACQ"
set CollateRegPrefix "Collate.Asp"

set Sharc1 "sharc1"
set Sharc2 "sharc2"
set Sharc3 "sharc3"

set [set Sharc1]Present 0 
set [set Sharc2]Present 0
set [set Sharc3]Present 0

set StatsNetVarName "DACQ_Stats"
set RatesNetVarName "DACQ_Rates"
set StatsRegName "StatsArray"
set RatesRegName "RatesArray"
set AttList "-tint -prw -ayes -i0 -fd"


proc custom_pre_make {} { global NUM; make-registers $NUM; }

proc custom_clear {} { inform SHARC do_command 7 }

proc find_sharc_servers {} {
   global Sharc1 Sharc2 Sharc3
   global sharc1Present sharc2Present sharc3Present
   global RDOUTserverList RDOUTserver

   set RDOUTserverList [inform sys inquire RDOUTserverList]
   set RDOUTserver [first $RDOUTserverList]

   set Sharc1 [inform SHARC inquire Sharc1]
   set Sharc2 [inform SHARC inquire Sharc2]
   set Sharc3 [inform SHARC inquire Sharc3]

   set sharc1Present [inform SHARC inquire sharc1Present]
   set sharc2Present [inform SHARC inquire sharc2Present]
   set sharc3Present [inform SHARC inquire sharc3Present]

   for {set asp 0} {$asp <= 5} {incr asp} {
      if {$asp <= 1} then {
         set RegisterServer $Sharc1
      } else {
         if {$asp <= 3} then {
            set RegisterServer $Sharc2
         } else {
            set RegisterServer $Sharc3
         }
      }
      if {[set [set RegisterServer]Present] == 1} then {
         lappend RDOUTserverList asp$asp
      }
   }

}

proc make-registers {num} {
   global RegisterServer RDOUTserverList EGreport
   global Sharc1 Sharc2 Sharc3
   global sharc1Present sharc2Present sharc3Present
   global AttList
   global StatsNetVarName RatesNetVarName
   global StatsRegName RatesRegName
   global MergeRegPrefix CollateRegPrefix

   find_sharc_servers

   set RegisterServer [first $RDOUTserverList]

   set EGr $EGreport
   set EGreport -1

   set RegPrefix $MergeRegPrefix
   set NetVarPrefix ""

   set attlist $AttList
   lappend attlist "-n$NetVarPrefix$StatsNetVarName -l$num"
   define-register generic $RegPrefix.$StatsRegName "$attlist"

   set attlist $AttList
   lappend attlist "-n$NetVarPrefix$RatesNetVarName -l$num"
   define-register generic $RegPrefix.$RatesRegName "$attlist"

   for {set asp 0} {$asp <= 5} {incr asp} {
      if {$asp <= 1} then {
         set RegisterServer $Sharc1
      } else {
         if {$asp <= 3} then {
            set RegisterServer $Sharc2
         } else {
            set RegisterServer $Sharc3
         }
      }
      if {[set [set RegisterServer]Present] == 1} then {
         set RegPrefix $CollateRegPrefix$asp
         set NetVarPrefix asp[set asp]_

         set attlist $AttList
         lappend attlist "-n$NetVarPrefix$StatsNetVarName -l$num"
         define-register generic $RegPrefix.$StatsRegName "$attlist"

         set attlist $AttList
         lappend attlist "-n$NetVarPrefix$RatesNetVarName -l$num"
         define-register generic $RegPrefix.$RatesRegName "$attlist"
      }
   }

 
   set EGreport $EGr
}

proc read-Stats {} {
  global RDOUTserver NUM
  global Sharc1 Sharc2 Sharc3
  global MergeRegPrefix CollateRegPrefix
  global StatsRegName RegisterServer

   set asp [string index $RDOUTserver end]
   if {[string equal -length 3 $RDOUTserver "asp"]} then {
      set RegPrefix $CollateRegPrefix$asp
      if {$asp <= 1} then {
         set RegisterServer $Sharc1
      } else {
         if {$asp <= 3} then {
            set RegisterServer $Sharc2
         } else {
            set RegisterServer $Sharc3
         }
      }
   } else {
      set RegisterServer $RDOUTserver
      set RegPrefix $MergeRegPrefix
   }

  set reg $RegPrefix.$StatsRegName
  write-attributes $reg "-l $NUM"
  set reg /$RegisterServer/$reg
  return [EG "register :name '$reg' ?value"]
}

proc read-Rates {} {
  global RDOUTserver NUM
  global Sharc1 Sharc2 Sharc3
  global MergeRegPrefix CollateRegPrefix
  global RatesRegName RegisterServer

   set asp [string index $RDOUTserver end]
   if {[string equal -length 3 $RDOUTserver "asp"]} then {
      set RegPrefix $CollateRegPrefix$asp
      set asp [string index $RDOUTserver end]
      if {$asp <= 1} then {
         set RegisterServer $Sharc1
      } else {
         if {$asp <= 3} then {
            set RegisterServer $Sharc2
         } else {
            set RegisterServer $Sharc3
         }
      }
   } else {
      set RegisterServer $RDOUTserver
      set RegPrefix $MergeRegPrefix
   }

  set reg $RegPrefix.$RatesRegName
  write-attributes $reg "-l $NUM"
  set reg /$RegisterServer/$reg
  return [EG "register :name '$reg' ?value"]
}

proc click-redisplay {} {
   global RDOUTserver NUM ITEM Stats Rates
   global EGreport

   set-footer "Updating..."
   set EGreport -1
   set-busy

   set Stats [read-Stats]
   for {set i 0} {$i < $NUM} {incr i} {
       if {[info exists ITEM($i)]} then {
	   :modify STAT$i :value [lindex $Stats $i]
       }
   }

   set Rates [read-Rates]
   for {set i 0} {$i < $NUM} {incr i} {
       if {[info exists ITEM($i)]} then {
	   :modify RATE$i :value [lindex $Rates $i]
       }
   }

   custom_redisplay

   clear-busy
   set EGreport 0
   set-footer "Counts Updated [date]"
}
