# 
# March 2001 -  GREAT TDR32V DA
#    updated for metronome2 and pattern register  feb 2004
#    updated for new DA system (SAGE)  feb 2008
#    updated for SAGE   jul 2009


set VERSION "GREAT"
set VARIANT "SAGE"

set env(RDOUThdwre) Merge
set env(RDOUTserver) merge

set DAProgram "Merge (master)"
if {[info exists env(MIDASBASE)]}  then {
  set LinuxDirectory  [file join $env(MIDASBASE) GREAT++ MergeServer]
} else {
  set LinuxDirectory  [file join /MIDAS GREAT++ MergeServer]
}

loadsource sys.common
loadsource lyrtech

insert-debug "RDOUTserver is $RDOUTserver"

set frametitle "MIDAS GREAT++ Experiment Control (TDR32V/LyrTech)"

set CAMACPresent 1
set VMEPresent 1
set VXIPresent 1

set CAMACEnable 1
set VMEEnable 1
set VXIEnable 1

set SAGEPresent 1
set SAGEEnable 0

set X2VBPresent 0

set RDOUTEnable 1

set TSPresent 1
set EBPresent 1

set TSEnable 0

set histPresent 1
set histEnable 1

set NamesPres 0
set Names {}

set txEnable 0
set ebtxEnable 0

set sagestate 0

#set Supported_Modules [list S9418 V785 V775 V792 V878 V879 V1190 V1290 V767 VME USER]
set Supported_Modules [list VME]

if {[info exists env(RDOUThdwre)] == 0} then {set hdwre Merge} else {set hdwre $env(RDOUThdwre)}
set RDOUThdwre $hdwre

set POLLTICKS 100000


set DelayTicks 6

#    append to common lists

lappend registerlist [list NetVar.DACQ.Chistmode	generic  "-nDACQ_HistMode -tint"]
lappend registerlist [list NetVar.DACQ.CPileupRate	generic  "-nDACQ_PileupRate -tint"]
lappend registerlist [list NetVar.DACQ.InitialTimeStamp	generic  "-nMerge_InitialTimeStamp -tuint"]
lappend registerlist [list NetVar.DACQ.Spaused          generic  "-nMerge_IsPaused -tint"]
lappend registerlist [list NetVar.DACQ.StatsArray       generic  "-n DACQ_Stats -t int -a yes -i 0 -l 64"]
lappend registerlist [list NetVar.DACQ.RatesArray       generic  "-n DACQ_Rates -t int -a yes -i 0 -l 64"]
lappend registerlist [list NetVar.DACQ.OutputTraces     generic  "-nMerge_OutputTrace -tint"]
lappend registerlist [list NetVar.DACQ.KeepNonMonotonic generic  "-nMerge_KeepNonMonotonic -tint"]
lappend registerlist [list NetVar.DACQ.DontUseSyncs     generic  "-nMerge_DontUseSyncs -tint"]
lappend registerlist [list NetVar.DACQ.TimeOrderWindow  generic  "-nMerge_TimeOrderWindow -tint"]
lappend registerlist [list NetVar.DACQ.GREATWordOrder   generic  "-nDACQ_GREATWordOrder -tint"]

lappend setuplist [list "Force Spectrum Map" "map_needed"]

set SetupMessage "\n\
Are you sure you wish to SETUP the data acquisition system?\n\n\
SETUP will STOP data acquisition.\n\
Configure the VXI Control software and restore VXI parameter settings.\n\
It will also recreate all histograms selected.\n\
This may take several minutes.\n\n\
If you decide to continue please wait for the SETUP sequence to complete and DO NOT touch the system while it is in progress.
"

proc Was_Set_Options {} {
    global Options

    load_Options

    Set_Common_Options

    update_Options
}


proc delete_spectra {} {
   global SpectrumServer
}

proc create_spectra {} {
    global SpectrumServer Options

   set range $Options(Stat.channels)
   set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $range :name 'Stat' !new)" } m]
   map_spectrum "Stat 0"

   set range $Options(Rate.channels)
   set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $range :name 'Rate' !new)" } m]
   
#     all other spectra created by the software histogram interface
    map_spectra
}

proc map_spectra {} {
   global EGreport env
   global RDOUTEnable

   if {$RDOUTEnable == 0} then {return}
   
   if {[map_?_needed] == 1} then {return}
   
   inform shist start
   inform shist close-frame

   set EGreport_SAVE $EGreport
   set EGreport -1
   
   set-footer "mapping spectra"
   set-busy

   set l [spectra-names]
   set n 0
   foreach item $l {
       set name [second $item]
       set Xname [inform shist Give_XlateR $name]
       set module [first [split $Xname .]]
       set adc    [second [split $Xname .]]
       set m [second [split $module #]]
       set a [string range $adc 3 end]
       if {"[first [split $module #]]" == "tdr32v"} then {
          map_spectrum "$name [expr ($m << 5) + $a]"
       }
   }

   map_not_needed

   set EGreport $EGreport_SAVE

   clear-busy
   set-footer "mapped [llength $l] spectra"
}

proc click-rdout  {v} {
    global POLLTICKS

   global RDOUThdwre RDOUTEnable; if {$RDOUTEnable == 0} then {return}

    set z [catch {
        if {$v==0} then {
#    Halt

              catch {inform SHARC :query_Mapped} isMapped
              inform SHARC do_command 1
              if {$isMapped!=1} {inform SHARC close-frame}
              
              after cancel Check-DA
              execute-command 1 "Halt"
              after 6000 inform MergeLinks display-Merge
        }
        if {$v==1} then {
#    Go
              map_spectra

              execute-command 2 "Go"
              after 5000 Check-DA

              catch {inform SHARC :query_Mapped} isMapped
              inform SHARC do_command 2
              if {$isMapped!=1} {inform SHARC close-frame}

              after 6000 inform MergeLinks display-Merge
        }
        if {$v==2} then {
#    Setup

              Poll_Ticks $POLLTICKS
              
              after cancel Check-DA
              execute-command 3 "Setup"

# We must do SHARC (Collator) Setup after Merge, since Merge sets up
# addresses to be used by the Collators
# but there's still a problem - timing - so we wait a bit
#
              after 2000
              catch {inform SHARC :query_Mapped} isMapped
              inform SHARC do_setup
              if {$isMapped!=1} {inform SHARC close-frame}
              inform MergeLinks do_setup
              catch {inform MergeLinks :query_Mapped} isMapped
              if {$isMapped!=1} {inform MergeLinks close-frame}
        }
        if {$v==3} then {
#    Reset

              catch {inform SHARC :query_Mapped} isMapped
              inform SHARC do_command 9
              if {$isMapped!=1} {inform SHARC close-frame}
              inform MergeLinks do_setup
              catch {inform MergeLinks :query_Mapped} isMapped
              if {$isMapped!=1} {inform MergeLinks close-frame}
              
              after cancel Check-DA
              execute-command 9 "Reset"
        }
        wait_for_completion
    } m ]
    redisplay-rdout
    clear-footer
}

proc redisplay-rdout {} {
   global RDOUTEnable

   if {$RDOUTEnable == 0} then {
      :modify RDOUT :value -1
      :modify PAUSE :value 0
      return
   }
   set readoutstate [expr [read-register NetVar.DACQ.Sstate]-1]
   :modify RDOUT :value $readoutstate

   set pause [read-register NetVar.DACQ.Spaused]
   :modify PAUSE :value $pause

   inform MergeLinks display-Merge
}

proc redisplay-camac {} {:modify CAMAC :value -1}

proc Custom_PostCheck {arg} {
    catch {inform SHARC inquire started} isStarted
    if {$isStarted} {
       catch {inform SHARC :query_Mapped} isMapped
    } else {
      set isMapped 0
    }
    inform SHARC start
    inform SHARC load_configuration
    if {$isMapped!=1} {inform SHARC close-frame}


    catch {inform MergeLinks inquire started} isStarted
    if {$isStarted} {
       catch {inform MergeLinks :query_Mapped} isMapped
    } else {
      set isMapped 0
    }
    inform MergeLinks start
    inform MergeLinks load_configuration
    if {$isMapped!=1}    {inform MergeLinks close-frame}

    if {$arg == 2} {                            ;#  setup
       inform metronome2 start
       inform metronome2 close-frame

       inform pattern_register start
       inform pattern_register close-frame

       inform tdr32v start
       inform tdr32v close-frame
    }
}

set my_opt -1

proc Custom_SetupD {option} {    ;#  opt = 0 setup where needed,  opt = 1  setup everything,  opt = 2  prompt
    global my_opt

    set my_opt $option
    upvar opt opt

    inform metronome2 click-calibrate
    set opt 1    ;# will force VXI parameter restore
}

proc Custom_PostSetup {opt} {
   global my_opt

   if {$my_opt == -1} {set my_opt $opt}
   LyrTech_Setup $my_opt
}

# command 16 is User Application 1  (Merge_start() in Merge DA code)
# tells the merge DA program to allow data through, starting with timestamp
# equal or greater than the metronome re-synch time
# Should be called after Go_Electronics which has invoked the metronome's
# synchronize-all to set up the initial timestamp value for the cards and
# written it to the merge program
proc Custom_PostGo {} {
   insert-log "START to Merge"
   click-special 16

   LyrTech_Go
}

proc Go_Electronics {} {

    insert-log "GO to Metronome2: STARX: 0"
    inform metronome2 set-register Start 1    ;#   removes system reset (starx)
    click-vme 4 ;# This calls a Custom_PostGo for the pattern registers
    inform metronome2 synchronize-all
}

proc Stop_Electronics {} {

    LyrTech_Stop

    insert-log "STOP to Metronome2: STARX: 1"
    inform metronome2 set-register Start 0    ;#   asserts system reset (starx)
}

proc Reset_Electronics {} {
# Don't force a reset (means action is currently a no-op)
    LyrTech_Reset 0
}

# command 17 is User Application 2  (Merge_pause() in Merge DA code)
# This is a toggle.
# if DA is outputting data this tells it to discard data (PAUSE state)
# if DA is discarding data this tells it to restart from the next good
# SYNC pulse timestamp. Otherwise do nothing.
proc click-pause {v} {
   set pause [read-register NetVar.DACQ.Spaused]
   if {$pause != $v} then {click-special 17}
   redisplay-rdout
   return  [read-register NetVar.DACQ.Spaused]
}



proc click-sage {w vl} {
#insert-debug "Clicked SAGE with widget $w, value $vl"
    switch $vl {
      0  {LyrTech_Stop}
      1  {LyrTech_Go}
      2  {LyrTech_Setup 1  ;# Setup everything}
      3  {LyrTech_Reset 1  ;# Force a reset}
    }
    redisplay-sage
}

proc click-sageenable {w vl} {
    global debug  SAGEEnable
    set SAGEEnable $vl
    redisplay-xxxenable
}

proc reshow-sage {} {
   global SAGEEnable sagestate

#  if {$SAGEEnable == 0} then {return}

   set-busy

   set sagestate -1
   catch {set sagestate [LyrTech_ReadState]}
   :modify SAGE :value $sagestate

   clear-busy
}

proc redisplay-sage {} {
   reshow-sage
   after 10000 reshow-sage
   after 15000 reshow-sage
}

# Same as in sys.common.tcl plus sage
proc redisplay-xxxenable {} {
  global CAMACEnable VMEEnable VXIEnable CAMACPresent VMEPresent VXIPresent RDOUTEnable
  global SAGEPresent SAGEEnable

 if {$CAMACPresent == 1} {
    :modify CAMACENABLE :value $CAMACEnable
    redisplay-camac
 }
 if {$VMEPresent == 1} {
    :modify VMEENABLE :value $VMEEnable
    redisplay-vme
 }
 if {$VXIPresent == 1} {
    :modify VXIENABLE :value $VXIEnable
    redisplay-vxi
 }
 if {[info exists SAGEPresent] && $SAGEPresent == 1} {
    :modify SAGEENABLE :value $SAGEEnable
    redisplay-sage
 }

    :modify RDOUTENABLE :value $RDOUTEnable
}

proc set-experiment args {LyrTech_Servers}
proc unset-experiment {} {NoLyrTech_Servers}
